# -*- coding: utf-8 -*-

from imio.smartweb.core.interfaces import IImioSmartwebCoreLayer
from imio.smartweb.core.testing import IMIO_SMARTWEB_CORE_FUNCTIONAL_TESTING
from imio.smartweb.core.testing import ImioSmartwebTestCase
from plone import api
from plone.api import portal as portal_api
from plone.app.testing import logout
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import TEST_USER_PASSWORD
from plone.app.textfield.value import RichTextValue
from plone.protect.authenticator import createToken
from plone.testing.zope import Browser
from zope.component import getMultiAdapter
from zope.component import queryMultiAdapter
from zope.interface import alsoProvides

import json
import mock
import re
import transaction


class TestSections(ImioSmartwebTestCase):
    layer = IMIO_SMARTWEB_CORE_FUNCTIONAL_TESTING

    def setUp(self):
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.page = api.content.create(
            container=self.portal,
            type="imio.smartweb.Page",
            id="page",
        )
        alsoProvides(self.request, IImioSmartwebCoreLayer)
        self.section_text = api.content.create(
            container=self.page,
            type="imio.smartweb.SectionText",
            title="Section Text",
            id="section-text",
        )
        self.section_text.text = RichTextValue(
            "<p>My rich text</p>", "text/html", "text/html"
        )
        api.content.transition(self.page, "publish")

    def test_htmx_not_loaded_for_anonymous(self):
        logout()
        view = queryMultiAdapter((self.page, self.request), name="full_view")
        self.assertNotIn("https://unpkg.com/htmx.org@1.9.10/dist/htmx.js", view())

    def test_all_html_elements_are_here(self):
        view = queryMultiAdapter((self.page, self.request), name="full_view")
        # test if htmx well loaded
        self.assertIn("https://unpkg.com/htmx.org@1.9.10/dist/htmx.js", view())
        section_text_uid = self.section_text.UID()
        # test if elements are here and have the right id
        self.assertIn(f"sizesForm_{section_text_uid}", view())
        self.assertIn(f'hx-boost="{section_text_uid}"', view())
        self.assertIn(f'id="selected_size_{section_text_uid}"', view())
        self.assertIn(f'hx-target="#selected_size_{section_text_uid}"', view())

    def test_options_in_select(self):
        view = queryMultiAdapter((self.page, self.request), name="full_view")
        section_text_view = getMultiAdapter(
            (self.section_text, self.request), name="view"
        )
        available_sizes = section_text_view.get_sizes
        match = re.search(
            r'<form class="form_section_size".*?</form>', view(), re.DOTALL
        )
        form_to_choose_size = match.group(0)
        nb_occurrences = len(re.findall(r"<option", form_to_choose_size))
        self.assertEqual(len(available_sizes), nb_occurrences)
        for size in available_sizes:
            self.assertIn(
                f'<option title="{size["value"]}" value="{size["key"]}" class="icon_{size["key"]}" >',
                form_to_choose_size,
            )
            # self.assertIn(
            #     f'<option value="{size["key"]}">{size["value"]}</option>',
            #     form_to_choose_size,
            # )

    def test_change_section_size(self):
        portal_api.get_current_language = mock.Mock(return_value="en")
        transaction.commit()
        section_text_uid = self.section_text.UID()
        browser = Browser(self.layer["app"])
        browser.addHeader(
            "Authorization",
            "Basic %s:%s"
            % (
                TEST_USER_NAME,
                TEST_USER_PASSWORD,
            ),
        )
        browser.open(f"{self.page.absolute_url()}/full_view/?language=fr")
        contents = browser.contents
        div_section_container = (
            '<div class="sortable-section sectiontext" data-id="section-text" style="">'
        )
        self.assertIn(div_section_container, contents)

        select = browser.getControl(name=f"select_{section_text_uid}")
        select.value = "col-sm-12"
        browser.getForm(name=f"sizesForm_{section_text_uid}").submit()
        section_text_view = getMultiAdapter(
            (self.section_text, self.request), name="view"
        )
        self.assertEqual(section_text_view.save_size, json.dumps({}))
        select_name = f"select_{self.section_text.UID()}"
        self.request.form[select_name] = select.value[0]
        self.request.form["_authenticator"] = createToken()
        section_text_view = getMultiAdapter(
            (self.section_text, self.request), name="view"
        )
        section_text_view.save_size
        self.assertEqual(
            section_text_view.save_size,
            json.dumps({"id": "col-sm-12", "title": "Full width"}),
        )
        transaction.commit()

        browser.open(f"{self.page.absolute_url()}/full_view/?language=fr")
        contents = browser.contents
        div_section_container = f'<div class="sortable-section sectiontext {select.value[0]}" data-id="section-text" style="">'
        self.assertIn(div_section_container, contents)



# -*- coding: utf-8 -*-

from imio.smartweb.core.utils import get_scale_url
from imio.smartweb.core.behaviors.minisite import IImioSmartwebMinisite
from plone import api
from plone.app.layout.viewlets.common import LogoViewlet as baseLogoViewlet
from plone.formwidget.namedfile.converter import b64decode_file
from plone.namedfile.utils import get_contenttype
from plone.registry.interfaces import IRegistry
from Products.CMFPlone.interfaces import ISiteSchema
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.component import getUtility


def is_svg(filename):
    content_type = get_contenttype(filename=filename)
    return content_type == "image/svg+xml"


class LogoViewlet(baseLogoViewlet):
    """ """

    index = ViewPageTemplateFile("logo.pt")

    show_logo = True
    show_title = False
    is_svg = False
    is_in_minisite = False

    def update(self):
        super(LogoViewlet, self).update()
        self.root = api.portal.get_navigation_root(self.context)
        logo_field = None
        if IImioSmartwebMinisite.providedBy(self.root):
            self.update_minisite()
            logo_field = self.root.logo
            data = getattr(logo_field, "data", None)
            filename = getattr(logo_field, "filename", None)
        else:
            registry = getUtility(IRegistry)
            settings = registry.forInterface(ISiteSchema, prefix="plone")
            if getattr(settings, "site_logo", False):
                logo_field = settings.site_logo
                filename, data = b64decode_file(logo_field)
        if not self.show_logo or not logo_field:
            return
        if is_svg(filename):
            self.is_svg = True
            self.svg_data = data

    def update_minisite(self):
        minisite = self.root
        self.is_in_minisite = True
        if minisite.logo is None:
            self.show_logo = False
        else:
            self.show_logo = minisite.logo_display_mode in ["logo", "logo_title"]
            self.img_src = get_scale_url(minisite, self.request, "logo", "preview")
        self.show_title = minisite.logo_display_mode in ["title", "logo_title"]
        self.logo_title = self.navigation_root_title



# -*- coding: utf-8 -*-

from imio.smartweb.core.contents import ICirkwiView
from imio.smartweb.core.tests.utils import get_html
from imio.smartweb.core.testing import IMIO_SMARTWEB_CORE_INTEGRATION_TESTING
from imio.smartweb.core.testing import ImioSmartwebTestCase
from plone import api
from plone.app.testing import login
from plone.app.testing import logout
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.textfield.value import RichTextValue
from plone.dexterity.interfaces import IDexterityFTI
from zope.component import createObject
from zope.component import queryMultiAdapter
from zope.component import queryUtility


import requests_mock

CIRKWI_API_GOOD_WIDGET_URL = "https://www.modulesbox.com/fr/api/module/12345"
CIRKWI_API_BAD_WIDGET_URL = "https://www.modulesbox.com/fr/api/module/123"


class TestCirkwiView(ImioSmartwebTestCase):
    layer = IMIO_SMARTWEB_CORE_INTEGRATION_TESTING

    def _changeUser(self, loginName):
        logout()
        login(self.portal, loginName)
        self.member = api.user.get_current()
        self.request["AUTHENTICATED_USER"] = self.member

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        portal_types = self.portal.portal_types
        parent_id = portal_types.constructContent(
            "imio.smartweb.Folder",
            self.portal,
            "parent_container",
            title="Parent container",
        )
        self.parent = self.portal[parent_id]
        self._changeUser("test")
        self.cirkwi_bad_widget_html = get_html("resources/cirkwi_bad_widget_mock.html")
        self.cirkwi_good_widget_html = get_html(
            "resources/cirkwi_good_widget_mock.html"
        )

    def test_ct_cirkwi_schema(self):
        fti = queryUtility(IDexterityFTI, name="imio.smartweb.CirkwiView")
        schema = fti.lookupSchema()
        self.assertEqual(ICirkwiView, schema)

    def test_ct_cirkwi_fti(self):
        fti = queryUtility(IDexterityFTI, name="imio.smartweb.CirkwiView")
        self.assertTrue(fti)

    def test_ct_cirkwi_factory(self):
        fti = queryUtility(IDexterityFTI, name="imio.smartweb.CirkwiView")
        factory = fti.factory
        obj = createObject(factory)

        self.assertTrue(
            ICirkwiView.providedBy(obj),
            "ICirkwiView not provided by {0}!".format(
                obj,
            ),
        )

    def test_ct_cirkwi_adding(self):
        setRoles(self.portal, TEST_USER_ID, ["Contributor"])
        obj = api.content.create(
            container=self.parent,
            type="imio.smartweb.CirkwiView",
            id="cirkwiview",
        )

        self.assertTrue(
            ICirkwiView.providedBy(obj),
            "ICirkwiView not provided by {0}!".format(
                obj.id,
            ),
        )

        parent = obj.__parent__
        self.assertIn("cirkwiview", parent.objectIds())

        # check that deleting the object works too
        api.content.delete(obj=obj)
        self.assertNotIn("cirkwiview", parent.objectIds())

    def test_ct_cirkwi_globally_addable(self):
        setRoles(self.portal, TEST_USER_ID, ["Contributor"])
        fti = queryUtility(IDexterityFTI, name="imio.smartweb.CirkwiView")
        self.assertTrue(fti.global_allow, "{0} is globally addable!".format(fti.id))

    @requests_mock.Mocker()
    def test_cirkwi_view(self, m):
        folder = api.content.create(
            container=self.portal,
            type="imio.smartweb.Folder",
            id="folder",
        )
        cirkwiview = api.content.create(
            container=folder,
            type="imio.smartweb.CirkwiView",
            id="cirkwiview",
        )
        m.get(CIRKWI_API_BAD_WIDGET_URL, status_code=404)
        cirkwiview.cirkwi_widget_id = "123"
        view = queryMultiAdapter((cirkwiview, self.request), name="view")
        self.assertIn('class="cirkwi_richtext"', view())
        self.assertIn('<div class="cirkwi_contents">404</div>', view())
        self.assertNotIn(
            "https://www.modulesbox.com/template/4/css/cirkwi/main.css", view()
        )

        m.get(CIRKWI_API_GOOD_WIDGET_URL, text=self.cirkwi_good_widget_html)
        cirkwiview.cirkwi_widget_id = "12345"
        view = queryMultiAdapter((cirkwiview, self.request), name="view")
        self.assertIn('class="cirkwi_richtext"', view())
        self.assertIn(
            "https://www.modulesbox.com/template/4/css/cirkwi/main.css", view()
        )
        self.assertNotIn("<p>My rich text</p>", view())

        cirkwiview.text = RichTextValue("<p>My rich text</p>", "text/html", "text/html")
        view = queryMultiAdapter((cirkwiview, self.request), name="view")
        self.assertIn("<p>My rich text</p>", view())



# -*- coding: utf-8 -*-

from plone import api
from plone.app.layout.viewlets.httpheaders import HeaderViewlet


class HtmxViewlet(HeaderViewlet):
    """htmx"""

    def update(self):
        super(HtmxViewlet, self).update()

    @property
    def is_anonymous(self):
        return api.user.is_anonymous()



# -*- coding: utf-8 -*-

from imio.smartweb.core.behaviors.subsite import IImioSmartwebSubsite
from imio.smartweb.core.behaviors.minisite import IImioSmartwebMinisite
from imio.smartweb.core.testing import IMIO_SMARTWEB_CORE_INTEGRATION_TESTING
from imio.smartweb.core.testing import ImioSmartwebTestCase
from imio.smartweb.core.viewlets.subsite import SubsiteLogoViewlet
from imio.smartweb.core.viewlets.subsite import SubsiteNavigationViewlet
from imio.smartweb.core.tests.utils import make_named_image
from plone import api
from plone.app.testing import TEST_USER_ID
from plone.app.testing import setRoles
from plone.dexterity.content import ASSIGNABLE_CACHE_KEY
from plone.namedfile.file import NamedBlobImage
from Products.Five.browser import BrowserView as View
from zope.component import getMultiAdapter
from zope.viewlet.interfaces import IViewlet
from zope.viewlet.interfaces import IViewletManager


class TestSubsite(ImioSmartwebTestCase):
    layer = IMIO_SMARTWEB_CORE_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests"""
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.folder = api.content.create(
            container=self.portal,
            type="imio.smartweb.Folder",
            title="Folder",
            id="folder",
        )
        # avoid cached empty value for instance behaviors
        delattr(self.request, ASSIGNABLE_CACHE_KEY)

    def test_activation(self):
        view = getMultiAdapter((self.portal, self.request), name="subsite_settings")
        self.assertFalse(view.available())

        view = getMultiAdapter((self.folder, self.request), name="subsite_settings")
        self.assertTrue(view.available())
        self.assertFalse(view.enabled())

        view.enable()
        self.assertTrue(IImioSmartwebSubsite.providedBy(self.folder))
        self.assertFalse(view.available())
        self.assertTrue(view.enabled())

        view.disable()
        self.assertFalse(IImioSmartwebSubsite.providedBy(self.folder))
        self.assertTrue(view.available())
        self.assertFalse(view.enabled())

        minisite_view = getMultiAdapter(
            (self.folder, self.request), name="minisite_settings"
        )
        minisite_view.enable()
        self.assertFalse(view.available())

    def test_viewlet_navigation(self):
        view = getMultiAdapter((self.folder, self.request), name="subsite_settings")
        view.enable()

        api.content.create(
            container=self.folder, type="imio.smartweb.Page", title="Page 1", id="page1"
        )
        subfolder = api.content.create(
            container=self.folder,
            type="imio.smartweb.Folder",
            title="Subfolder",
            id="subfolder",
        )
        api.content.create(
            container=subfolder,
            type="imio.smartweb.Page",
            title="Sub Page 1",
            id="subpage1",
        )

        viewlet = SubsiteNavigationViewlet(self.portal, self.request, None, None)
        viewlet.update()
        self.assertFalse(viewlet.available())

        viewlet = SubsiteNavigationViewlet(self.folder, self.request, None, None)
        viewlet.update()
        self.assertIn("Sub Page 1", viewlet.render_globalnav())

        self.folder.menu_depth = 1
        viewlet = SubsiteNavigationViewlet(self.folder, self.request, None, None)
        viewlet.update()
        self.assertTrue(viewlet.available())
        self.assertEqual(len(viewlet.portal_tabs), 2)

        viewlet = SubsiteNavigationViewlet(subfolder, self.request, None, None)
        viewlet.update()
        self.assertTrue(viewlet.available())
        self.assertEqual(viewlet.subsite_root, self.folder)
        self.assertEqual(len(viewlet.portal_tabs), 2)
        self.assertEqual(
            viewlet.render_globalnav(),
            '<li class="nav_page1 nav-item"><a href="http://nohost/plone/folder/page1" class="state-private nav-link">Page 1</a></li><li class="nav_subfolder nav-item"><a href="http://nohost/plone/folder/subfolder" class="state-private nav-link">Subfolder</a></li>',
        )
        self.assertNotIn("Sub Page 1", viewlet.render_globalnav())

    def test_viewlet_logo(self):
        view = getMultiAdapter((self.folder, self.request), name="subsite_settings")
        view.enable()
        viewlet = SubsiteLogoViewlet(self.folder, self.request, None, None)
        viewlet.update()
        self.assertTrue(viewlet.available())
        self.assertTrue(viewlet.show_title())
        self.assertFalse(viewlet.show_logo())

        view = View(self.folder, self.request)
        manager = getMultiAdapter(
            (self.folder, self.request, view),
            IViewletManager,
            name="plone.portalheader",
        )
        viewlet = getMultiAdapter(
            (self.folder, self.request, view, manager),
            IViewlet,
            name="imio.smartweb.subsite_logo",
        )
        self.assertIn("Folder", viewlet())
        self.folder.logo_display_mode = "logo"
        self.assertFalse(viewlet.show_title())
        self.assertFalse(viewlet.show_logo())
        self.folder.logo_display_mode = "logo_title"
        self.assertTrue(viewlet.show_title())
        self.assertFalse(viewlet.show_logo())
        self.folder.logo = NamedBlobImage(**make_named_image())
        self.assertTrue(viewlet.show_logo())
        self.folder.logo_display_mode = "logo"
        self.assertTrue(viewlet.show_logo())

        # Title should remain the same on sub-contents
        subfolder = api.content.create(
            container=self.folder,
            type="imio.smartweb.Folder",
            title="Subfolder",
        )
        view = View(subfolder, self.request)
        manager = getMultiAdapter(
            (subfolder, self.request, view), IViewletManager, name="plone.portalheader"
        )
        viewlet = getMultiAdapter(
            (subfolder, self.request, view, manager),
            IViewlet,
            name="imio.smartweb.subsite_logo",
        )
        self.assertIn("Folder", viewlet())
        self.assertNotIn("Subfolder", viewlet())

    def test_subsite_in_subsite(self):
        view = getMultiAdapter((self.folder, self.request), name="subsite_settings")
        view.enable()
        self.assertTrue(IImioSmartwebSubsite.providedBy(self.folder))
        subsite2 = api.content.create(
            container=self.folder,
            type="imio.smartweb.Folder",
            title="subsite2",
            id="subsite2",
        )
        view = getMultiAdapter((subsite2, self.request), name="subsite_settings")
        view.enable()
        self.assertTrue(IImioSmartwebSubsite.providedBy(subsite2))

    def test_subsite_in_minisite(self):
        view = getMultiAdapter((self.folder, self.request), name="minisite_settings")
        view.enable()
        self.assertTrue(IImioSmartwebMinisite.providedBy(self.folder))

        subsite_view = getMultiAdapter(
            (self.folder, self.request), name="subsite_settings"
        )
        self.assertFalse(subsite_view.available())

        subsite = api.content.create(
            container=self.folder,
            type="imio.smartweb.Folder",
            title="subsite",
            id="subsite",
        )
        view = getMultiAdapter((subsite, self.request), name="subsite_settings")
        view.enable()
        self.assertTrue(IImioSmartwebSubsite.providedBy(subsite))

    def test_cannot_enable_subsite_on_minisite(self):
        minisite_view = getMultiAdapter(
            (self.folder, self.request), name="minisite_settings"
        )
        minisite_view.enable()
        subsite_view = getMultiAdapter(
            (self.folder, self.request), name="subsite_settings"
        )
        self.assertFalse(subsite_view.available())
        self.assertFalse(subsite_view.enabled())
        subsite_view.enable()
        self.assertFalse(subsite_view.enabled())

    def test_subsite_body_class(self):
        subfolder = api.content.create(
            container=self.folder,
            type="imio.smartweb.Folder",
            title="Subfolder",
            id="subfolder",
        )
        page = api.content.create(
            container=subfolder,
            type="imio.smartweb.Page",
            title="Subpage",
            id="subpage",
        )

        view = getMultiAdapter((subfolder, self.request), name="subsite_settings")
        view.enable()
        template = subfolder.restrictedTraverse("view")
        layout_view = page.restrictedTraverse("@@plone_layout")
        body_class = layout_view.bodyClass(template, view)
        self.assertIn("is-in-subsite", body_class)

        view = getMultiAdapter((subfolder, self.request), name="subsite_settings")
        view.disable()
        layout_view = page.restrictedTraverse("@@plone_layout")
        body_class = layout_view.bodyClass(template, view)
        self.assertNotIn("is-in-subsite", body_class)



# -*- coding: utf-8 -*-
from collective.anysurfer.layout import LayoutPolicy as AnysurferLayoutPolicy
from eea.facetednavigation.config import ANNO_FACETED_LAYOUT
from eea.facetednavigation.subtypes.interfaces import IFacetedNavigable
from imio.smartweb.core.behaviors.minisite import IImioSmartwebMinisite
from imio.smartweb.core.viewlets.banner import BannerViewlet
from imio.smartweb.core.viewlets.subsite import BaseSubsiteViewlet
from plone import api
from plone.app.layout.globals import layout as base
from zope.annotation.interfaces import IAnnotations


class LayoutPolicy(AnysurferLayoutPolicy):
    """
    Enhanced layout policy
    """

    def bodyClass(self, template, view):
        """
        Extend Plone to add a CSS class on <body> based on :
         1. banner image
         2. minisite
         3. subsite
         4. faceted layout
        """
        context = self.context
        request = self.request

        # Get default body classes
        body_class = base.LayoutPolicy.bodyClass(self, template, view)

        # 1. Add bloc-banner class if there is a banner on the page
        banner_viewlet = BannerViewlet(context, request, view)
        if banner_viewlet.available() and not banner_viewlet.is_banner_locally_hidden:
            body_class += " bloc-banner"

        # 2. Add is-in-minisite class to body if we are in a minisite
        root = api.portal.get_navigation_root(context)
        if IImioSmartwebMinisite.providedBy(root):
            body_class += " is-in-minisite"

        # 3. Add is-in-subsite class to body if we are in a subsite
        subsite_viewlet = BaseSubsiteViewlet(context, request, view)
        if subsite_viewlet.available():
            body_class += " is-in-subsite"

        # 4. Add faceted class to body if a faceted layout is define
        if IFacetedNavigable.providedBy(context):
            faceted_layout = IAnnotations(self.context).get(
                ANNO_FACETED_LAYOUT, "faceted-block-view"
            )
            body_class += f" {faceted_layout}"
        return body_class



# -*- coding: utf-8 -*-

from plone.app.layout.viewlets.httpheaders import HTTPCachingHeaders


class MyHTTPCachingHeaders(HTTPCachingHeaders):
    def update(self):
        super(MyHTTPCachingHeaders, self).update()

    def getHeaders(self):
        result = super(MyHTTPCachingHeaders, self).getHeaders()
        result.append(("prefix", "og: http://ogp.me/ns#"))
        return result



# -*- coding: utf-8 -*-

from imio.smartweb.core.utils import get_scale_url
from imio.smartweb.core.contents.sections.views import SectionView


class TextView(SectionView):
    """Gallery Section view"""

    def get_scale_url(self, item):
        request = self.request
        scale = self.context.image_size
        if self.context.image._height > self.context.image._width:
            return get_scale_url(item, request, "image", scale, "portrait")
        return get_scale_url(item, request, "image", scale, "paysage")



# -*- coding: utf-8 -*-

from plone.gallery.views.photo_gallery import PhotoGallery
from imio.smartweb.core.utils import get_scale_url
from imio.smartweb.core.contents.sections.views import SectionView


class GalleryView(SectionView, PhotoGallery):
    """Gallery Section view"""

    def get_scale_url(self, item, scale, orientation="paysage"):
        request = self.request
        return get_scale_url(item, request, "image", scale, orientation)



# -*- coding: utf-8 -*-

from collective.messagesviewlet.utils import get_messages_to_show
from collective.messagesviewlet.browser.messagesviewlet import GlobalMessagesViewlet
from imio.smartweb.core.behaviors.minisite import IImioSmartwebMinisite
from plone import api


class MessagesViewlet(GlobalMessagesViewlet):
    def getAllMessages(self):
        context = self.context
        root = api.portal.get_navigation_root(context)
        if not IImioSmartwebMinisite.providedBy(root):
            return super(MessagesViewlet, self).getAllMessages()
        messages = get_messages_to_show(self.context)
        messages = [m for m in messages if m.location in self.location_filter]
        root_absolute_url = root.absolute_url()
        messages = [
            m for m in messages if m.absolute_url().startswith(root_absolute_url)
        ]
        return messages



# -*- coding: utf-8 -*-

from imio.smartweb.locales import SmartwebMessageFactory as _
from plone.autoform.interfaces import IFormFieldProvider
from plone.supermodel import model
from zope import schema
from zope.interface import provider


@provider(IFormFieldProvider)
class IOrientation(model.Schema):
    model.fieldset("layout", label=_("Layout"), fields=["orientation"])
    orientation = schema.Choice(
        title=_("Orientation"),
        required=True,
        vocabulary="imio.smartweb.vocabulary.Orientation",
        default="paysage",
    )



# -*- coding: utf-8 -*-

from imio.smartweb.core.utils import get_json
from plone import api
from plone.dexterity.content import Container
from plone.rest import Service
from urllib.parse import urlencode
from zope.interface import implementer
from zope.interface import Interface

import json


class BaseEndpoint(object):
    language = "fr"
    remote_endpoint = ""

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def __call__(self):
        results = get_json(self.query_url, timeout=20)
        return results

    @property
    def query_url(self):
        raise NotImplementedError

    def convert_cached_image_scales(
        self,
        item,
        modified_hash,
        field="image",
        scales=["vignette", "affiche"],
        orientation="paysage",
    ):
        """Remove image from result dict and add generated image scales URLs
        with cache key"""
        for scale in scales:
            scale_name = "_".join(filter(None, [orientation, scale]))
            cached_scale_url = (
                f"{item['@id']}/@@images/{field}/{scale_name}?cache_key={modified_hash}"
            )
            item[f"{field}_{scale}_scale"] = cached_scale_url
        item[f"{field}_full_scale"] = (
            f"{item['@id']}/@@images/{field}/?cache_key={modified_hash}"
        )
        del item[field]

    def construct_query_string(self, params):
        params = "&".join(params)
        form = self.request.form
        extra_params = []
        for k, v in form.items():
            if isinstance(v, list):
                # handles multiple metadata_fields
                for value in v:
                    extra_params.append((k, value))
            else:
                extra_params.append((k, v))
        current_lang = api.portal.get_current_language()[:2]
        if current_lang != "fr":
            extra_params.append(("translated_in_{}".format(current_lang), 1))
        extra_params = urlencode(extra_params)
        return "&".join(filter(None, [params, extra_params]))


class BaseService(Service):
    def render(self):
        response = self.request.response
        response.setHeader("Content-type", "application/json")
        content = self.reply()
        return json.dumps(
            content,
            indent=2,
            separators=(", ", ": "),
        )


class IRestView(Interface):
    """"""


@implementer(IRestView)
class RestView(Container):
    """Shared base class for REST views contents"""



# -*- coding: utf-8 -*-

from imio.smartweb.core.config import EVENTS_URL
from imio.smartweb.core.contents.rest.base import BaseEndpoint
from imio.smartweb.core.contents.rest.base import BaseService
from imio.smartweb.core.utils import hash_md5
from plone.restapi.interfaces import IExpandableElement
from zope.component import adapter
from zope.interface import implementer
from zope.interface import Interface


class BaseEventsEndpoint(BaseEndpoint):
    def __call__(self):
        results = super(BaseEventsEndpoint, self).__call__()
        if not results or not results.get("items"):
            return results
        orientation = self.context.orientation
        for result in results["items"]:
            modified_hash = hash_md5(result["modified"])
            if result.get("image"):
                self.convert_cached_image_scales(
                    result, modified_hash, orientation=orientation
                )
            for sub_content in result.get("items", []):
                if sub_content["@type"] != "Image":
                    continue
                self.convert_cached_image_scales(
                    sub_content, modified_hash, "image", ["preview"], ""
                )
        return results

    @property
    def query_url(self):
        params = [
            "selected_agendas={}".format(self.context.selected_agenda),
            "metadata_fields=category",
            "metadata_fields=local_category",
            "metadata_fields=container_uid",
            "metadata_fields=topics",
            "metadata_fields=start",
            "metadata_fields=end",
            "metadata_fields=has_leadimage",
            "metadata_fields=UID",
            "sort_on=event_dates",
            "fullobjects=1",
            "b_size={}".format(self.context.nb_results),
        ]
        if self.context.selected_event_types is not None:
            for event_type in self.context.selected_event_types:
                params.append(f"event_type={event_type}")
        params = self.construct_query_string(params)
        url = f"{EVENTS_URL}/{self.remote_endpoint}?{params}"
        return url


@implementer(IExpandableElement)
@adapter(Interface, Interface)
class EventsEndpoint(BaseEventsEndpoint):
    remote_endpoint = "@events"


@implementer(IExpandableElement)
@adapter(Interface, Interface)
class EventsFiltersEndpoint(BaseEventsEndpoint):
    remote_endpoint = "@search-filters"


class EventsEndpointGet(BaseService):
    def reply(self):
        return EventsEndpoint(self.context, self.request)()


class EventsFiltersEndpointGet(BaseService):
    def reply(self):
        return EventsFiltersEndpoint(self.context, self.request)()



# -*- coding: utf-8 -*-

from imio.smartweb.core.config import DIRECTORY_URL
from imio.smartweb.core.contents.rest.base import BaseEndpoint
from imio.smartweb.core.contents.rest.base import BaseService
from imio.smartweb.core.utils import hash_md5
from plone import api
from plone.restapi.interfaces import IExpandableElement
from zope.component import adapter
from zope.interface import implementer
from zope.interface import Interface


class BaseDirectoryEndpoint(BaseEndpoint):
    def __call__(self):
        results = super(BaseDirectoryEndpoint, self).__call__()
        if not results.get("items"):
            return results
        orientation = self.context.orientation
        for result in results["items"]:
            modified_hash = hash_md5(result["modified"])
            if result.get("image"):
                self.convert_cached_image_scales(
                    result, modified_hash, orientation=orientation
                )
            if result.get("logo"):
                self.convert_cached_image_scales(
                    result, modified_hash, "logo", ["thumb"], ""
                )
            for sub_content in result.get("items", []):
                if sub_content["@type"] != "Image":
                    continue
                self.convert_cached_image_scales(
                    sub_content, modified_hash, "image", ["preview"], ""
                )
        return results

    @property
    def query_url(self):
        entity_uid = api.portal.get_registry_record("smartweb.directory_entity_uid")
        params = [
            "selected_entities={}".format(entity_uid),
            "portal_type=imio.directory.Contact",
            "metadata_fields=facilities",
            "metadata_fields=taxonomy_contact_category",
            "metadata_fields=topics",
            "metadata_fields=has_leadimage",
            "fullobjects=1",
            "sort_on=sortable_title",
            "b_size={}".format(self.context.nb_results),
        ]
        if self.context.selected_categories is not None:
            for category in self.context.selected_categories:
                params.append(f"taxonomy_contact_category.query={category}")
            params.append("taxonomy_contact_category.operator=or")
        params = self.construct_query_string(params)
        url = f"{DIRECTORY_URL}/{self.remote_endpoint}?{params}"
        return url


@implementer(IExpandableElement)
@adapter(Interface, Interface)
class DirectoryEndpoint(BaseDirectoryEndpoint):
    remote_endpoint = "@search"


@implementer(IExpandableElement)
@adapter(Interface, Interface)
class DirectoryFiltersEndpoint(BaseDirectoryEndpoint):
    remote_endpoint = "@search-filters"


class DirectoryEndpointGet(BaseService):
    def reply(self):
        return DirectoryEndpoint(self.context, self.request)()


class DirectoryFiltersEndpointGet(BaseService):
    def reply(self):
        return DirectoryFiltersEndpoint(self.context, self.request)()



# -*- coding: utf-8 -*-

from imio.smartweb.core.contents.sections.base import ISection
from imio.smartweb.core.contents.sections.base import Section
from imio.smartweb.locales import SmartwebMessageFactory as _
from plone.supermodel import model
from zope.interface import implementer
from zope import schema


class ISectionGallery(ISection):
    """Marker interface and Dexterity Python Schema for SectionGallery"""

    model.fieldset("layout", fields=["image_scale"])
    image_scale = schema.Choice(
        title=_("Image scale for images"),
        default="affiche",
        vocabulary="imio.smartweb.vocabulary.Scales",
        required=True,
    )


@implementer(ISectionGallery)
class SectionGallery(Section):
    """SectionGallery class"""

    manage_content = True



from .blocks.link.content import BlockLink  # NOQA
from .blocks.link.content import IBlockLink  # NOQA
from .pages.pages import IDefaultPages  # NOQA
from .pages.pages import IPages  # NOQA
from .pages.pages import Pages  # NOQA
from .folder.content import Folder  # NOQA
from .folder.content import IFolder  # NOQA
from .pages.cirkwi.content import CirkwiView  # NOQA
from .pages.cirkwi.content import ICirkwiView  # NOQA
from .pages.footer.content import Footer  # NOQA
from .pages.footer.content import IFooter  # NOQA
from .pages.herobanner.content import HeroBanner  # NOQA
from .pages.herobanner.content import IHeroBanner  # NOQA
from .pages.page.content import IPage  # NOQA
from .pages.page.content import Page  # NOQA
from .pages.portal_page.content import IPortalPage  # NOQA
from .pages.portal_page.content import PortalPage  # NOQA
from .pages.procedure.content import IProcedure  # NOQA
from .pages.procedure.content import Procedure  # NOQA
from .rest.base import RestView  # NOQA
from .rest.base import IRestView  # NOQA
from .rest.directory.content import DirectoryView  # NOQA
from .rest.directory.content import IDirectoryView  # NOQA
from .rest.events.content import EventsView  # NOQA
from .rest.events.content import IEventsView  # NOQA
from .rest.news.content import INewsView  # NOQA
from .rest.news.content import NewsView  # NOQA
from .sections.base import ISection  # NOQA
from .sections.base import Section  # NOQA
from .sections.collection.content import ISectionCollection  # NOQA
from .sections.collection.content import SectionCollection  # NOQA
from .sections.contact.content import ISectionContact  # NOQA
from .sections.contact.content import SectionContact  # NOQA
from .sections.events.content import ISectionEvents  # NOQA
from .sections.events.content import SectionEvents  # NOQA
from .sections.external_content.content import ISectionExternalContent  # NOQA
from .sections.external_content.content import SectionExternalContent  # NOQA
from .sections.files.content import ISectionFiles  # NOQA
from .sections.files.content import SectionFiles  # NOQA
from .sections.gallery.content import ISectionGallery  # NOQA
from .sections.gallery.content import SectionGallery  # NOQA
from .sections.html.content import ISectionHTML  # NOQA
from .sections.html.content import SectionHTML  # NOQA
from .sections.links.content import ISectionLinks  # NOQA
from .sections.links.content import SectionLinks  # NOQA
from .sections.map.content import ISectionMap  # NOQA
from .sections.map.content import SectionMap  # NOQA
from .sections.news.content import ISectionNews  # NOQA
from .sections.news.content import SectionNews  # NOQA
from .sections.postit.content import ISectionPostit  # NOQA
from .sections.postit.content import SectionPostit  # NOQA
from .sections.selections.content import ISectionSelections  # NOQA
from .sections.selections.content import SectionSelections  # NOQA
from .sections.sendinblue.content import ISectionSendinblue  # NOQA
from .sections.sendinblue.content import SectionSendinblue  # NOQA
from .sections.slide.content import ISectionSlide  # NOQA
from .sections.slide.content import SectionSlide  # NOQA
from .sections.text.content import ISectionText  # NOQA
from .sections.text.content import SectionText  # NOQA
from .sections.video.content import ISectionVideo  # NOQA
from .sections.video.content import SectionVideo  # NOQA



# -*- coding: utf-8 -*-

from imio.smartweb.core.contents.pages.views import PagesView
from imio.smartweb.locales import SmartwebMessageFactory as _
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile


class ProcedureView(PagesView):
    index = ViewPageTemplateFile("../view.pt")

    @property
    def no_items_message(self):
        return _("There is no section on this procedure.")



# -*- coding: utf-8 -*-

from plone.app.contentmenu.menu import DisplaySubMenuItem
from plone.memoize.instance import memoize


class CollectionDisplaySubMenuItem(DisplaySubMenuItem):
    @memoize
    def available(self):
        if self.disabled():
            return False
        return True



# -*- coding: utf-8 -*-

from imio.smartweb.core.interfaces import IHtmxViewUtils
from Products.Five.browser import BrowserView
from zope.interface import implementer


@implementer(IHtmxViewUtils)
class NoTitleView(BrowserView):
    def __call__(self):
        return ""



from BTrees.OOBTree import OOBTree
from imio.smartweb.core.config import DIRECTORY_URL
from imio.smartweb.core.config import EVENTS_URL
from imio.smartweb.core.config import NEWS_URL
from imio.smartweb.core.contents.rest.utils import get_auth_sources_response
from imio.smartweb.core.contents.rest.utils import get_entity_id
from plone import api
from plone.base.interfaces import IPloneSiteRoot
from plone.app.layout.sitemap.sitemap import SiteMapView
from plone.registry.interfaces import IRegistry
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import normalizeString
from zope.component import getUtility

import logging

logger = logging.getLogger("imio.smartweb.core")


class CustomSiteMapView(SiteMapView):

    def objects(self):
        """Returns the data to create the sitemap."""

        friendlytypes = [
            "Link",
            "imio.smartweb.Folder",
            "LIF",
            "LRF",
            "imio.smartweb.Page",
            "imio.smartweb.Procedure",
            "File",
            "Collection",
            "imio.smartweb.PortalPage",
            "Image",
            "imio.smartweb.CirkwiView",
        ]

        catalog = getToolByName(self.context, "portal_catalog")
        query = {}
        utils = getToolByName(self.context, "plone_utils")
        query["portal_type"] = utils.getUserFriendlyTypes(friendlytypes)
        registry = getUtility(IRegistry)
        typesUseViewActionInListings = frozenset(
            registry.get("plone.types_use_view_action_in_listings", [])
        )

        is_plone_site_root = IPloneSiteRoot.providedBy(self.context)
        if not is_plone_site_root:
            query["path"] = "/".join(self.context.getPhysicalPath())

        query["is_default_page"] = True
        default_page_modified = OOBTree()
        for item in catalog.searchResults(query):
            key = item.getURL().rsplit("/", 1)[0]
            value = (item.modified.micros(), item.modified.ISO8601())
            default_page_modified[key] = value

        # The plone site root is not catalogued.
        if is_plone_site_root:
            loc = self.context.absolute_url()
            date = self.context.modified()
            # Comparison must be on GMT value
            modified = (date.micros(), date.ISO8601())
            default_modified = default_page_modified.get(loc, None)
            if default_modified is not None:
                modified = max(modified, default_modified)
            lastmod = modified[1]
            yield {
                "loc": loc,
                "lastmod": lastmod,
                # 'changefreq': 'always',
                #  hourly/daily/weekly/monthly/yearly/never
                # 'prioriy': 0.5, # 0.0 to 1.0
            }

        query["is_default_page"] = False
        for item in catalog.searchResults(query):
            loc = item.getURL()
            date = item.modified
            # Comparison must be on GMT value
            modified = (date.micros(), date.ISO8601())
            default_modified = default_page_modified.get(loc, None)
            if default_modified is not None:
                modified = max(modified, default_modified)
            lastmod = modified[1]
            if item.portal_type in typesUseViewActionInListings:
                loc += "/view"
            yield {
                "loc": loc,
                "lastmod": lastmod,
            }

        auth_sources = {
            "directory": {
                "entity_reg_var": "smartweb.directory_entity_uid",
                "main_rest_view": "smartweb.default_directory_view",
                "vocabulary": "imio.smartweb.vocabulary.RemoteDirectoryEntities",
                "url": DIRECTORY_URL,
            },
            "events": {
                "entity_reg_var": "smartweb.events_entity_uid",
                "main_rest_view": "smartweb.default_events_view",
                "vocabulary": "imio.smartweb.vocabulary.RemoteEventsEntities",
                "url": EVENTS_URL,
            },
            "news": {
                "entity_reg_var": "smartweb.news_entity_uid",
                "main_rest_view": "smartweb.default_news_view",
                "vocabulary": "imio.smartweb.vocabulary.RemoteNewsEntities",
                "url": NEWS_URL,
            },
        }
        for auth_source_key, auth_source_value in auth_sources.items():
            entity_uid = api.portal.get_registry_record(
                auth_source_value.get("entity_reg_var")
            )
            if entity_uid is None:
                return self.request.response.setStatus(404, "No entity found")
            entity_id = get_entity_id(auth_source_value.get("vocabulary"), entity_uid)
            auth_source_uid = api.portal.get_registry_record(
                f"smartweb.default_{auth_source_key}_view", default=None
            )
            if auth_source_uid is None:
                return self.request.response.setStatus(
                    404, "No default authentic source found"
                )
            obj = api.content.get(UID=auth_source_uid)
            if obj is None:
                logger.warning(
                    f"Seems that a main authentic view (for {auth_source_key}) is not found"
                )
                continue
            auth_source_view_url = obj.absolute_url()
            results = get_auth_sources_response(
                auth_source_key, normalizeString(entity_id), (60 * 60 * 24)
            ).json()
            if results is None or results.get("type") == "ValueError":
                continue
            for item in results.get("items"):
                item_id = normalizeString(item.get("title"))
                item_uid = item.get("id")
                loc = f"{auth_source_view_url}/{item_id}?u={item_uid}"
                lastmod = item.get("modified")
                yield {
                    "loc": loc,
                    "lastmod": lastmod,
                }



# -*- coding: utf-8 -*-

from plone import api
from imio.smartweb.core.testing import IMIO_SMARTWEB_CORE_INTEGRATION_TESTING
from imio.smartweb.core.testing import ImioSmartwebTestCase
from imio.smartweb.core.browser.dashboards.plausible import PlausibleView
from unittest import mock
from zope.component import queryMultiAdapter

import os


class Testplausible(ImioSmartwebTestCase):
    layer = IMIO_SMARTWEB_CORE_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests"""
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]

    def set_registry_records(self):
        api.portal.set_registry_record("smartweb.plausible_site", "site-registry.be")
        api.portal.set_registry_record("smartweb.plausible_token", "token-registry")
        api.portal.set_registry_record("smartweb.plausible_url", "url-registry.be")

    @mock.patch.dict(
        os.environ,
        {
            "SMARTWEB_PLAUSIBLE_SITE": "",
            "SMARTWEB_PLAUSIBLE_TOKEN": "",
            "SMARTWEB_PLAUSIBLE_URL": "",
        },
    )
    def test_noenv(self):
        view = PlausibleView(self.portal, self.request)
        self.assertFalse(view.is_plausible_set)
        self.set_registry_records()
        self.assertTrue(view.is_plausible_set)
        self.assertEqual(
            view.get_iframe_src,
            "https://url-registry.be/share/site-registry.be?auth=token-registry&embed=true&theme=light&background=transparent",
        )
        self.assertEqual(
            view.get_embedhostjs_src,
            "https://url-registry.be/js/embed.host.js",
        )

    @mock.patch.dict(
        os.environ,
        {
            "SMARTWEB_PLAUSIBLE_SITE": "site-varenv.be",
            "SMARTWEB_PLAUSIBLE_TOKEN": "token-varenv",
            "SMARTWEB_PLAUSIBLE_URL": "url-varenv.be",
        },
    )
    def test_env(self):
        view = PlausibleView(self.portal, self.request)
        self.assertTrue(view.is_plausible_set)
        self.assertEqual(
            view.get_iframe_src,
            "https://url-varenv.be/share/site-varenv.be?auth=token-varenv&embed=true&theme=light&background=transparent",
        )
        self.assertEqual(
            view.get_embedhostjs_src,
            "https://url-varenv.be/js/embed.host.js",
        )
        self.set_registry_records()
        self.assertEqual(
            view.get_iframe_src,
            "https://url-varenv.be/share/site-varenv.be?auth=token-varenv&embed=true&theme=light&background=transparent",
        )
        self.assertEqual(
            view.get_embedhostjs_src,
            "https://url-varenv.be/js/embed.host.js",
        )

    @mock.patch.dict(
        os.environ,
        {
            "SMARTWEB_PLAUSIBLE_SITE": "",
            "SMARTWEB_PLAUSIBLE_TOKEN": "",
            "SMARTWEB_PLAUSIBLE_URL": "",
        },
    )
    def test_plausible_view(self):
        view = queryMultiAdapter((self.portal, self.request), name="stats")
        self.assertNotIn("iframe", view())
        self.assertIn("Plausible analytics is not set", view())
        self.set_registry_records()
        self.assertIn("iframe", view())
        self.assertNotIn("Plausible analytics is not set", view())



# -*- coding: utf-8 -*-

from imio.smartweb.common.utils import rich_description
from imio.smartweb.core.contents.sections.views import SectionView
from imio.smartweb.core.utils import batch_results


class PostitView(SectionView):
    """ """

    def items(self):
        items = self.context.postits or []
        results = []
        for item in items:
            description = rich_description(item["description"])
            results.append(
                {
                    "title": item["title"],
                    "subtitle": item["subtitle"],
                    "description": description,
                    "url": "",
                    "icon": None,
                    "has_icon": False,
                    "image": "",
                    "has_image": False,
                    "open_in_new_tab": False,
                }
            )
        return batch_results(results, self.context.nb_results_by_batch)



# -*- coding: utf-8 -*-

from imio.smartweb.core.contents.sections.views import SectionView

import json


class MapView(SectionView):
    """ """

    def data_geojson(self):
        """Return the context geolocation as GeoJSON string."""
        coordinates = self.context.geolocation
        if not coordinates:
            return
        geo_json = {
            "type": "Feature",
            "properties": {},
            "geometry": {
                "type": "Point",
                "coordinates": [coordinates.longitude, coordinates.latitude],
            },
        }
        return json.dumps(geo_json)



# -*- coding: utf-8 -*-

from freezegun import freeze_time
from imio.smartweb.core.testing import IMIO_SMARTWEB_CORE_INTEGRATION_TESTING
from imio.smartweb.core.testing import ImioSmartwebTestCase
from imio.smartweb.core.tests.utils import make_named_image
from imio.smartweb.core.utils import batch_results
from imio.smartweb.core.utils import get_scale_url
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.namedfile.file import NamedBlobImage
from plone.uuid.interfaces import IUUID


class TestUtils(ImioSmartwebTestCase):
    layer = IMIO_SMARTWEB_CORE_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests"""
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

    def test_batch_results(self):
        lst = [1, 2, 3, 4, 5, 6]
        self.assertEqual(batch_results(lst, 3), [[1, 2, 3], [4, 5, 6]])
        lst = [1, 2, 3, 4, 5, 6, 7]
        self.assertEqual(batch_results(lst, 3), [[1, 2, 3], [4, 5, 6], [7]])

    @freeze_time("2021-09-14 8:00:00")
    def test_get_scale_url(self):
        content = api.content.create(
            container=self.portal,
            type="imio.smartweb.Page",
            id="page",
        )
        self.assertEqual(
            get_scale_url(content, self.request, "image", "preview", ""), ""
        )
        uuid = IUUID(content)
        brain = api.content.find(UID=uuid)[0]
        self.assertEqual(get_scale_url(brain, self.request, "image", "preview", ""), "")

        content.image = NamedBlobImage(**make_named_image())
        content.reindexObject()
        self.assertIn(
            "http://nohost/plone/page/@@images/image-400-",
            get_scale_url(content, self.request, "image", "preview"),
        )
        paysage_scale = get_scale_url(
            content, self.request, "image", "affiche", "paysage"
        )
        portrait_scale = get_scale_url(
            content, self.request, "image", "affiche", "portrait"
        )
        self.assertIn("http://nohost/plone/page/@@images/image-750-", paysage_scale)
        self.assertIn("http://nohost/plone/page/@@images/image-448-", portrait_scale)
        self.assertNotEqual(paysage_scale, portrait_scale)
        self.assertEqual(
            get_scale_url(content, self.request, "image", "nonexisting"), ""
        )
        self.assertEqual(
            get_scale_url(content, self.request, "nonexisting", "preview"), ""
        )
        self.assertEqual(
            get_scale_url(content, self.request, "image", ""),
            "http://nohost/plone/page/@@images/image/?cache_key=78fd1bab198354b6877aed44e2ea0b4d",
        )
        brain = api.content.find(UID=uuid)[0]
        self.assertEqual(
            get_scale_url(brain, self.request, "image", "preview"),
            "http://nohost/plone/page/@@images/image/preview?cache_key=78fd1bab198354b6877aed44e2ea0b4d",
        )
        self.assertEqual(
            get_scale_url(brain, self.request, "image", "affiche", "paysage"),
            "http://nohost/plone/page/@@images/image/paysage_affiche?cache_key=78fd1bab198354b6877aed44e2ea0b4d",
        )
        self.assertEqual(
            get_scale_url(brain, self.request, "image", "affiche", "portrait"),
            "http://nohost/plone/page/@@images/image/portrait_affiche?cache_key=78fd1bab198354b6877aed44e2ea0b4d",
        )
        self.assertEqual(
            get_scale_url(brain, self.request, "image", "nonexisting"),
            "http://nohost/plone/page/@@images/image/nonexisting?cache_key=78fd1bab198354b6877aed44e2ea0b4d",
        )
        self.assertEqual(
            get_scale_url(brain, self.request, "image", ""),
            "http://nohost/plone/page/@@images/image/?cache_key=78fd1bab198354b6877aed44e2ea0b4d",
        )
        self.assertEqual(
            get_scale_url(brain, self.request, "image", "portrait_affiche", "portrait"),
            "http://nohost/plone/page/@@images/image/portrait_affiche?cache_key=78fd1bab198354b6877aed44e2ea0b4d",
        )



# -*- coding: utf-8 -*-

from imio.smartweb.core.behaviors.minisite import IImioSmartwebMinisite
from plone import api
from plone.app.layout.viewlets import common
from urllib.parse import urlparse


class MinisitePortalLinkViewlet(common.ViewletBase):
    """ """

    def get_hostname(self):
        root_url = api.portal.get_navigation_root(self.context).absolute_url()
        domain = urlparse(root_url).netloc
        if domain.startswith("www."):
            domain = domain[4:]
        return domain

    def available(self):
        root = api.portal.get_navigation_root(self.context)
        return IImioSmartwebMinisite.providedBy(root)



# # -*- coding: utf-8 -*-

from plone import api
from plone.app.multilingual import api as api_lng
from Products.Five.browser import BrowserView


def get_translation(obj, lang):
    if not api_lng.is_translatable(obj):
        return None
    translation_manager = api_lng.get_translation_manager(obj)
    translated_obj = translation_manager.get_translation(lang)
    return translated_obj


class RestViewBaseView(BrowserView):

    def get_rest_view_object(self):
        rest_view_uid = api.portal.get_registry_record(self.registry_key, default=None)
        if rest_view_uid is None:
            return None
        brains = api.content.find(UID=rest_view_uid)
        if brains:
            return brains[0].getObject()
        return None

    def __call__(self):
        language = self.request.form.get("language", "fr")
        default_language = api.portal.get_default_language()
        rest_view_obj = self.get_rest_view_object()
        if rest_view_obj is None:
            self.request.response.redirect(api.portal.get().absolute_url())
            return ""
        if default_language != language:
            translated_obj = get_translation(rest_view_obj, language)
            if translated_obj is not None:
                rest_view_obj = translated_obj
        rest_view_url = rest_view_obj.absolute_url()
        self.request.response.redirect(rest_view_url)
        return ""


class DirectoryView(RestViewBaseView):
    registry_key = "smartweb.default_directory_view"


class EventsView(RestViewBaseView):
    registry_key = "smartweb.default_events_view"


class NewsView(RestViewBaseView):
    registry_key = "smartweb.default_news_view"



# -*- coding: utf-8 -*-

from bs4 import BeautifulSoup
from imio.smartweb.core.testing import IMIO_SMARTWEB_CORE_FUNCTIONAL_TESTING
from imio.smartweb.core.testing import ImioSmartwebTestCase
from imio.smartweb.core.viewlets.navigation import ImprovedGlobalSectionsViewlet
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.dexterity.content import ASSIGNABLE_CACHE_KEY
from z3c.relationfield import RelationValue
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.intid.interfaces import IIntIds


class TestNavigation(ImioSmartwebTestCase):
    layer = IMIO_SMARTWEB_CORE_FUNCTIONAL_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.folder = api.content.create(
            container=self.portal,
            type="imio.smartweb.Folder",
            title="Folder",
            id="folder",
        )
        # avoid cached empty value for instance behaviors
        delattr(self.request, ASSIGNABLE_CACHE_KEY)

    def test_remove_subsites_children(self):
        subfolder = api.content.create(
            container=self.folder,
            type="imio.smartweb.Folder",
            title="Subfolder",
            id="subfolder",
        )
        api.content.create(
            container=subfolder,
            type="imio.smartweb.Page",
            title="Subpage",
            id="subpage",
        )
        viewlet = ImprovedGlobalSectionsViewlet(self.portal, self.request, None, None)
        viewlet.update()
        self.assertEqual(len(viewlet.navtree), 3)
        self.assertEqual(len(viewlet.navtree["/plone/folder"]), 1)
        self.assertIn("subfolder", viewlet.render_globalnav())
        self.assertIn("subpage", viewlet.render_globalnav())
        view = getMultiAdapter((self.folder, self.request), name="subsite_settings")
        view.enable()
        viewlet.remove_subsites_children()
        self.assertEqual(len(viewlet.navtree), 1)
        self.assertEqual(len(viewlet.navtree["/plone/folder"]), 0)
        self.assertNotIn("subfolder", viewlet.render_globalnav())
        self.assertNotIn("subpage", viewlet.render_globalnav())

    def test_remove_minisite(self):
        subfolder = api.content.create(
            container=self.folder,
            type="imio.smartweb.Folder",
            title="Subfolder",
            id="subfolder",
        )
        api.content.create(
            container=subfolder,
            type="imio.smartweb.Page",
            title="Subpage",
            id="subpage",
        )
        viewlet = ImprovedGlobalSectionsViewlet(self.portal, self.request, None, None)
        viewlet.update()
        self.assertEqual(len(viewlet.navtree), 3)
        self.assertEqual(len(viewlet.navtree["/plone/folder"]), 1)
        self.assertIn("subfolder", viewlet.render_globalnav())
        self.assertIn("subpage", viewlet.render_globalnav())
        view = getMultiAdapter((self.folder, self.request), name="minisite_settings")
        view.enable()
        viewlet.remove_minisites()
        self.assertEqual(len(viewlet.navtree), 1)
        self.assertEqual(len(viewlet.navtree["/plone/folder"]), 0)
        self.assertNotIn("subfolder", viewlet.render_globalnav())
        self.assertNotIn("subpage", viewlet.render_globalnav())

    def test_quick_accesses(self):
        api.portal.set_registry_record("plone.navigation_depth", 4)
        subfolder = api.content.create(
            container=self.folder,
            type="imio.smartweb.Folder",
            title="Subfolder level 2",
        )
        subsubfolder = api.content.create(
            container=subfolder,
            type="imio.smartweb.Folder",
            title="Subfolder level 3",
        )
        api.content.create(
            container=subsubfolder,
            type="imio.smartweb.Folder",
            title="Subfolder level 4 (to display submenu)",
        )
        page = api.content.create(
            container=self.portal,
            type="imio.smartweb.Page",
            title="Quick Page everywhere",
        )
        page_sub = api.content.create(
            container=self.portal,
            type="imio.smartweb.Page",
            title="Quick Page Sub",
        )
        page_sub_sub = api.content.create(
            container=self.portal,
            type="imio.smartweb.Page",
            title="Quick Page Su-Sub",
        )
        viewlet = ImprovedGlobalSectionsViewlet(self.portal, self.request, None, None)
        viewlet.update()
        self.assertNotIn('<li class="quick-access">', viewlet.render_globalnav())

        # Quick access on first level folder
        intids = getUtility(IIntIds)
        self.folder.quick_access_items = [
            RelationValue(intids.getId(page)),
        ]
        self.folder.reindexObject()
        html = viewlet.render_globalnav()
        self.assertIn('<li class="quick-access">', html)
        self.assertEqual(html.count('<li class="quick-access">'), 1)

        soup = BeautifulSoup(html)
        qa = soup.find("li", {"class": "nav_folder"}).find(
            "li", {"class": "quick-access"}
        )

        self.assertEqual(len(qa.find_all("li")), 1)
        self.assertIsNotNone(qa.find("li", {"class": "nav_quick-page-everywhere"}))

        # Quick access on second level folder
        intids = getUtility(IIntIds)
        subfolder.quick_access_items = [
            RelationValue(intids.getId(page)),
            RelationValue(intids.getId(page_sub)),
        ]
        subfolder.reindexObject()
        html = viewlet.render_globalnav()
        self.assertEqual(html.count('<li class="quick-access">'), 2)

        soup = BeautifulSoup(html)
        qa = soup.find("li", {"class": "nav_subfolder-level-2"}).find(
            "li", {"class": "quick-access"}
        )

        self.assertEqual(len(qa.find_all("li")), 2)
        self.assertIsNotNone(qa.find("li", {"class": "nav_quick-page-sub"}))

        # Quick access on third level folder
        intids = getUtility(IIntIds)
        subsubfolder.quick_access_items = [
            RelationValue(intids.getId(page)),
            RelationValue(intids.getId(page_sub_sub)),
        ]
        subsubfolder.reindexObject()
        html = viewlet.render_globalnav()
        self.assertEqual(html.count('<li class="quick-access">'), 3)

        soup = BeautifulSoup(html)
        qa = soup.find("li", {"class": "nav_subfolder-level-3"}).find(
            "li", {"class": "quick-access"}
        )

        self.assertEqual(len(qa.find_all("li")), 2)
        self.assertIsNotNone(qa.find("li", {"class": "nav_quick-page-su-sub"}))



# -*- coding: utf-8 -*-

from imio.smartweb.core.testing import IMIO_SMARTWEB_CORE_INTEGRATION_TESTING
from imio.smartweb.core.testing import ImioSmartwebTestCase
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from unittest.mock import patch
from zope.component import queryMultiAdapter

import json


class TestVocabulary(ImioSmartwebTestCase):
    layer = IMIO_SMARTWEB_CORE_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests"""
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.page = api.content.create(
            container=self.portal,
            type="imio.smartweb.Page",
            title="My Page",
        )

    def test_getvocabulary(self):
        self.request.form = {
            "name": "imio.smartweb.vocabulary.Topics",
            "field": "topics",
        }
        view = queryMultiAdapter((self.page, self.request), name="getVocabulary")
        result = json.loads(view())
        self.assertEqual(result["results"][0]["id"], "entertainment")
        self.assertEqual(result["results"][0]["text"], "Entertainment")
        with patch("plone.api.portal.get_current_language", return_value="fr"):
            view = queryMultiAdapter((self.page, self.request), name="getVocabulary")
            result = json.loads(view())
            self.assertEqual(result["results"][0]["id"], "entertainment")
            self.assertEqual(
                result["results"][0]["text"], "Activités et divertissement"
            )



# -*- coding: utf-8 -*-

from plone.app.layout.viewlets import common
from zope.component import getMultiAdapter
from zope.viewlet.interfaces import IViewletManager


class HeaderActionsViewlet(common.ViewletBase):
    def update(self):
        context_state = getMultiAdapter(
            (self.context, self.request), name="plone_context_state"
        )
        self.actions = context_state.actions("header_actions")


class HeaderActionsContainerViewlet(common.ViewletBase):
    """Viewlet containing the header actions viewlets manager"""


class IHeaderActionsViewletsManager(IViewletManager):
    """Viewlet manager containing header actions viewlets"""



# -*- coding: utf-8 -*-

from imio.smartweb.core.contents.sections.views import SECTION_ITEMS_HASH_KEY
from imio.smartweb.core.testing import IMIO_SMARTWEB_CORE_FUNCTIONAL_TESTING
from imio.smartweb.core.testing import ImioSmartwebTestCase
from imio.smartweb.core.tests.utils import get_json
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from time import sleep
from z3c.relationfield import RelationValue
from zope.annotation.interfaces import IAnnotations
from zope.component import getUtility
from zope.component import queryMultiAdapter
from zope.intid.interfaces import IIntIds

import json
import requests_mock


class TestSectionNews(ImioSmartwebTestCase):
    layer = IMIO_SMARTWEB_CORE_FUNCTIONAL_TESTING

    def setUp(self):
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.portalpage = api.content.create(
            container=self.portal,
            type="imio.smartweb.PortalPage",
            id="Portal page",
        )
        self.json_news = get_json("resources/json_rest_news.json")
        self.json_specific_news = get_json("resources/json_rest_specific_news.json")
        self.rest_news_view = api.content.create(
            container=self.portal,
            type="imio.smartweb.NewsView",
            title="Rest news view",
        )
        self.news = api.content.create(
            container=self.portalpage,
            type="imio.smartweb.SectionNews",
            title="My news",
        )

    @requests_mock.Mocker()
    def test_news(self, m):
        intids = getUtility(IIntIds)
        self.news.related_news = "64f4cbee9a394a018a951f6d94452914"
        self.news.linking_rest_view = RelationValue(intids.getId(self.rest_news_view))
        self.news.link_text = "Voir toutes les actualités"
        view = queryMultiAdapter((self.portalpage, self.request), name="full_view")
        self.assertIn("My news", view())
        news_view = queryMultiAdapter((self.news, self.request), name="carousel_view")
        self.assertEqual(news_view.items, [])
        url = "http://localhost:8080/Plone/@search?selected_news_folders=64f4cbee9a394a018a951f6d94452914&portal_type=imio.news.NewsItem&metadata_fields=container_uid&metadata_fields=category_title&metadata_fields=local_category&metadata_fields=topics&metadata_fields=has_leadimage&metadata_fields=modified&metadata_fields=effective&metadata_fields=UID&sort_limit=6&translated_in_en=1&sort_on=effective&sort_order=descending"
        m.get(url, text=json.dumps(self.json_news))
        self.assertEqual(news_view.items[0][0].get("title"), "Première actualité")
        self.assertEqual(len(news_view.items[0]), 3)

        self.news.specific_related_newsitems = [
            "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
            "bfe2b4391a0f4a8db6d8b7fed63d1c4a",
        ]
        url = "http://localhost:8080/Plone/@search?UID=aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa&UID=bfe2b4391a0f4a8db6d8b7fed63d1c4a&portal_type=imio.news.NewsItem&metadata_fields=container_uid&metadata_fields=category_title&metadata_fields=local_category&metadata_fields=topics&metadata_fields=has_leadimage&metadata_fields=modified&metadata_fields=effective&metadata_fields=UID&sort_limit=6&translated_in_en=1"
        m.get(url, text=json.dumps(self.json_specific_news))
        self.assertEqual(len(news_view.items[0]), 2)
        self.assertEqual(
            news_view.items[0][0].get("title"), "Restauration de la piscine"
        )

        self.news.specific_related_newsitems = [
            "bfe2b4391a0f4a8db6d8b7fed63d1c4a",
            "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
        ]
        m.get(url, text=json.dumps(self.json_specific_news))
        self.assertEqual(
            news_view.items[0][0].get("title"), "Restauration de la Bibliothèque"
        )

    @requests_mock.Mocker()
    def test_news_modified(self, m):
        intids = getUtility(IIntIds)
        self.news.related_news = "64f4cbee9a394a018a951f6d94452914"
        self.news.linking_rest_view = RelationValue(intids.getId(self.rest_news_view))
        annotations = IAnnotations(self.news)
        self.assertIsNone(annotations.get(SECTION_ITEMS_HASH_KEY))

        news_view = queryMultiAdapter((self.news, self.request), name="carousel_view")
        url = "http://localhost:8080/Plone/@search?selected_news_folders=64f4cbee9a394a018a951f6d94452914&portal_type=imio.news.NewsItem&metadata_fields=container_uid&metadata_fields=category_title&metadata_fields=local_category&metadata_fields=topics&metadata_fields=has_leadimage&metadata_fields=modified&metadata_fields=effective&metadata_fields=UID&sort_limit=6&translated_in_en=1&sort_on=effective&sort_order=descending"
        m.get(url, text=json.dumps(self.json_news))
        self.assertEqual(len(news_view.items[0]), 3)
        hash_1 = annotations.get(SECTION_ITEMS_HASH_KEY)
        self.assertIsNotNone(hash_1)
        first_modification = self.portalpage.ModificationDate()

        sleep(1)
        url = "http://localhost:8080/Plone/@search?selected_news_folders=64f4cbee9a394a018a951f6d94452914&portal_type=imio.news.NewsItem&metadata_fields=container_uid&metadata_fields=category_title&metadata_fields=local_category&metadata_fields=topics&metadata_fields=has_leadimage&metadata_fields=modified&metadata_fields=effective&metadata_fields=UID&sort_limit=6&translated_in_en=1&sort_on=effective&sort_order=descending"
        m.get(url, text="{}")
        self.assertEqual(len(news_view.items), 0)
        next_modification = self.portalpage.ModificationDate()
        hash_2 = annotations.get(SECTION_ITEMS_HASH_KEY)
        self.assertNotEqual(hash_1, hash_2)
        self.assertNotEqual(first_modification, next_modification)

        sleep(1)
        self.assertEqual(len(news_view.items), 0)
        last_modification = self.portalpage.ModificationDate()
        hash_3 = annotations.get(SECTION_ITEMS_HASH_KEY)
        self.assertEqual(hash_2, hash_3)
        self.assertEqual(next_modification, last_modification)

    @requests_mock.Mocker()
    def test_orientation(self, m):
        intids = getUtility(IIntIds)
        self.news.related_news = "64f4cbee9a394a018a951f6d94452914"
        self.news.linking_rest_view = RelationValue(intids.getId(self.rest_news_view))
        news_view = queryMultiAdapter((self.news, self.request), name="carousel_view")
        url = "http://localhost:8080/Plone/@search?selected_news_folders=64f4cbee9a394a018a951f6d94452914&portal_type=imio.news.NewsItem&metadata_fields=container_uid&metadata_fields=category_title&metadata_fields=local_category&metadata_fields=topics&metadata_fields=has_leadimage&metadata_fields=modified&metadata_fields=effective&metadata_fields=UID&sort_limit=6&translated_in_en=1&sort_on=effective&sort_order=descending"
        m.get(url, text=json.dumps(self.json_news))

        self.assertIn("paysage_vignette", news_view.items[0][0]["image"])
        self.news.orientation = "portrait"
        self.assertIn("portrait_vignette", news_view.items[0][0]["image"])

    @requests_mock.Mocker()
    def test_show_categories_or_topics(self, m):
        intids = getUtility(IIntIds)
        self.news.related_news = "64f4cbee9a394a018a951f6d94452914"
        self.news.linking_rest_view = RelationValue(intids.getId(self.rest_news_view))
        news_view = queryMultiAdapter((self.news, self.request), name="carousel_view")
        url = "http://localhost:8080/Plone/@search?selected_news_folders=64f4cbee9a394a018a951f6d94452914&portal_type=imio.news.NewsItem&metadata_fields=container_uid&metadata_fields=category_title&metadata_fields=local_category&metadata_fields=topics&metadata_fields=has_leadimage&metadata_fields=modified&metadata_fields=effective&metadata_fields=UID&sort_limit=6&translated_in_en=1&sort_on=effective&sort_order=descending"
        m.get(url, text=json.dumps(self.json_news))
        self.assertEqual(news_view.items[0][0]["category"], "Presse")
        self.news.show_categories_or_topics = "category"
        self.assertEqual(news_view.items[0][0]["category"], "Presse")
        self.news.show_categories_or_topics = "topic"
        self.assertEqual(news_view.items[0][0]["category"], "Education")
        self.news.show_categories_or_topics = ""
        self.assertEqual(news_view.items[0][0]["category"], "")



# -*- coding: utf-8 -*-

from freezegun import freeze_time
from gzip import GzipFile
from io import BytesIO
from imio.smartweb.core.testing import IMIO_SMARTWEB_CORE_FUNCTIONAL_TESTING
from imio.smartweb.core.testing import ImioSmartwebTestCase
from imio.smartweb.core.tests.utils import get_json
from imio.smartweb.core.tests.utils import make_named_image
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.textfield.value import RichTextValue
from plone.base.utils import safe_text
from plone.namedfile.file import NamedBlobImage
from unittest.mock import patch
from zope.component import getMultiAdapter

import json
import requests_mock


class TestPage(ImioSmartwebTestCase):
    layer = IMIO_SMARTWEB_CORE_FUNCTIONAL_TESTING

    @freeze_time("2024-02-02 8:00:00")
    def setUp(self):
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        api.portal.set_registry_record("plone.enable_sitemap", True)

        self.default_page = api.content.create(
            container=self.portal,
            type="imio.smartweb.PortalPage",
            title="Portal page",
            id="portal-page",
        )
        self.portal.setDefaultPage("portal-page")

        self.folder = api.content.create(
            container=self.portal,
            type="imio.smartweb.Folder",
            title="Folder",
            id="folder",
        )
        self.page = api.content.create(
            container=self.folder,
            type="imio.smartweb.Page",
            title="Page 1",
            id="page1",
        )
        self.section_text = api.content.create(
            container=self.page,
            type="imio.smartweb.SectionText",
            title="Section text",
        )
        self.section_text.text = RichTextValue(
            "<p>Kamoulox</p>", "text/html", "text/html"
        )

        self.rest_directory = api.content.create(
            container=self.portal,
            type="imio.smartweb.DirectoryView",
            title="directory view",
        )
        self.rest_agenda = api.content.create(
            container=self.portal,
            type="imio.smartweb.EventsView",
            title="agenda view",
        )
        self.rest_agenda.selected_agenda = "64f4cbee9a394a018a951f6d94452914"
        self.rest_news = api.content.create(
            container=self.portal,
            type="imio.smartweb.NewsView",
            title="news view",
        )
        self.rest_news.selected_news_folder = "64f4cbee9a394a018a951f6d94452914"
        api.content.transition(self.rest_directory, "publish")
        api.content.transition(self.rest_agenda, "publish")
        api.content.transition(self.rest_news, "publish")

    # 'http://localhost:8080/Plone/@search?selected_entities=396907b3b1b04a97896b12cc792c77f8&portal_type=imio.directory.Contact&fullobjects=0&sort_on=sortable_title'
    @freeze_time("2024-02-02 10:00:00")
    @requests_mock.Mocker()
    def test_sitemap(self, m):
        sitemap = getMultiAdapter(
            (self.portal, self.portal.REQUEST), name="sitemap.xml.gz"
        )
        xml = self.uncompress(sitemap())
        self.assertIn("<lastmod >2024-02-02T08:00:00", xml)
        self.assertIn("<loc>http://nohost/plone/folder</loc>", xml)
        self.assertIn("http://nohost/plone/folder/page1", xml)
        self.assertNotIn(
            "<loc>http://nohost/plone/folder/page1/gallery/image/view</loc>", xml
        )

        # Gallery and image created 2024-02-02 10:00:00
        gallery = api.content.create(
            container=self.page,
            type="imio.smartweb.SectionGallery",
            title="Gallery",
        )
        image = api.content.create(
            container=gallery,
            type="Image",
            title="Image",
        )
        image.image = NamedBlobImage(**make_named_image())
        sitemap = getMultiAdapter(
            (self.portal, self.portal.REQUEST), name="sitemap.xml.gz"
        )
        xml = self.uncompress(sitemap())
        self.assertIn(
            "<loc>http://nohost/plone/folder/page1/gallery/image/view</loc>", xml
        )
        # Gallery and image created 2024-02-02 10:00:00
        self.assertIn(
            "<loc>http://nohost/plone/folder/page1/gallery/image/view</loc>\n    <lastmod >2024-02-02T10:00:00",
            xml,
        )

        rest_views = {
            "directory": self.rest_directory,
            "events": self.rest_agenda,
            "news": self.rest_news,
        }
        contact_search_url = news_search_url = events_search_url = (
            "http://localhost:8080/Plone/@querystring-search"
        )
        for k, v in rest_views.items():
            api.portal.set_registry_record(f"smartweb.default_{k}_view", v.UID())
        with patch(
            "imio.smartweb.core.contents.rest.utils.get_wca_token",
            return_value="kamoulox",
        ):
            self.json_contacts = get_json("resources/json_contacts_raw_mock.json")
            m.post(contact_search_url, text=json.dumps(self.json_contacts))
            sitemap = getMultiAdapter(
                (self.portal, self.portal.REQUEST), name="sitemap.xml.gz"
            )
            xml = self.uncompress(sitemap())
            self.assertIn("<loc>http://nohost/plone/directory-view/contact1-title", xml)

        with patch(
            "imio.smartweb.core.contents.rest.utils.get_wca_token",
            return_value="kamoulox",
        ):
            self.json_news = get_json("resources/json_rest_news.json")
            m.post(news_search_url, text=json.dumps(self.json_news))
            sitemap = getMultiAdapter(
                (self.portal, self.portal.REQUEST), name="sitemap.xml.gz"
            )
            xml = self.uncompress(sitemap())
            self.assertIn("<loc>http://nohost/plone/news-view/", xml)

        with patch(
            "imio.smartweb.core.contents.rest.utils.get_wca_token",
            return_value="kamoulox",
        ):
            self.json_events = get_json("resources/json_rest_events.json")
            m.post(events_search_url, text=json.dumps(self.json_events))
            sitemap = getMultiAdapter(
                (self.portal, self.portal.REQUEST), name="sitemap.xml.gz"
            )
            xml = self.uncompress(sitemap())
            self.assertIn("<loc>http://nohost/plone/agenda-view/", xml)

    def uncompress(self, sitemapdata):
        sio = BytesIO(sitemapdata)
        unzipped = GzipFile(fileobj=sio)
        xml = unzipped.read()
        unzipped.close()
        return safe_text(xml)



# -*- coding: utf-8 -*-

from collective.anysurfer.interfaces import ILayerSpecific
from collective.messagesviewlet.interfaces import ICollectiveMessagesviewletLayer
from collective.solr.browser.interfaces import IThemeSpecific
from imio.smartweb.common.interfaces import IImioSmartwebCommonLayer
from plone.app.contenttypes.interfaces import IPloneAppContenttypesLayer
from zope.interface import Interface


class ISmartwebIcon(Interface):
    """Marker interface to easily find all registered icons in registry."""


class IViewWithoutLeadImage(Interface):
    """Marker interface for views that shouldn't show leadimage."""


class IImioSmartwebCoreLayer(
    IImioSmartwebCommonLayer,
    IPloneAppContenttypesLayer,
    ILayerSpecific,
    IThemeSpecific,
    ICollectiveMessagesviewletLayer,
):
    """Marker interface that defines a browser layer."""


class IArcgisViewUtils(Interface):
    """ """

    def get_portal_item_id():
        """Return id of a map"""


class IOgpViewUtils(Interface):
    """ """


class IOdwbViewUtils(Interface):
    """ """

    def is_odwb_widget_plugins():
        """Return True if we are in the generic odwb widget plugins"""


class IHtmxViewUtils(Interface):
    """ """



# -*- coding: utf-8 -*-

from Products.CMFPlone.interfaces.siteroot import IPloneSiteRoot
from collective.instancebehavior import disable_behaviors
from collective.instancebehavior import enable_behaviors
from imio.smartweb.core.behaviors.minisite import IImioSmartwebMinisite
from imio.smartweb.core.behaviors.minisite import IImioSmartwebMinisiteSettings
from imio.smartweb.core.behaviors.subsite import IImioSmartwebSubsite
from imio.smartweb.core.contents import IFolder
from imio.smartweb.locales import SmartwebMessageFactory as _
from plone import api
from plone.app.multilingual.interfaces import ILanguageRootFolder
from Products.Five.browser import BrowserView
from zope.interface import alsoProvides
from zope.interface import noLongerProvides


class MiniSiteSettings(BrowserView):
    """Subsite settings"""

    def enable(self):
        if not self.available():
            return
        enable_behaviors(
            self.context,
            [IImioSmartwebMinisiteSettings.__identifier__],
            [IImioSmartwebMinisiteSettings],
        )
        alsoProvides(self.context, IImioSmartwebMinisite)
        self.context.exclude_from_nav = True
        self.context.reindexObject(idxs=["object_provides", "exclude_from_nav"])
        api.portal.show_message(
            _("Minisite has been successfully activated"), self.request
        )
        self.request.response.redirect(self.context.absolute_url())

    def disable(self):
        disable_behaviors(
            self.context,
            [IImioSmartwebMinisiteSettings.__identifier__],
            [IImioSmartwebMinisiteSettings],
        )
        noLongerProvides(self.context, IImioSmartwebMinisite)
        self.context.exclude_from_nav = False
        self.context.reindexObject(idxs=["object_provides", "exclude_from_nav"])
        api.portal.show_message(_("Minisite has been disabled"), self.request)
        self.request.response.redirect(self.context.absolute_url())

    def available(self):
        if IPloneSiteRoot.providedBy(self.context):
            # PloneSite can't become minisite
            return False
        if ILanguageRootFolder.providedBy(self.context):
            # LRF can't become minisite
            return False
        parent = self.context.aq_parent
        if not IPloneSiteRoot.providedBy(parent) and not ILanguageRootFolder.providedBy(
            parent
        ):
            # Minisite can only be added in PloneSite / LRF
            return False
        if IImioSmartwebSubsite.providedBy(self.context):
            # Subsite can't be converted in minisite
            return False
        return IFolder.providedBy(self.context) and not self.enabled()

    def enabled(self):
        return IImioSmartwebMinisite.providedBy(self.context)



# -*- coding: utf-8 -*-

from imio.smartweb.core.contents.rest.view import BaseRestView
from imio.smartweb.core.interfaces import IOgpViewUtils
from plone import api
from zope.interface import implementer


@implementer(IOgpViewUtils)
class EventsViewView(BaseRestView):
    """EventsView view"""

    @property
    def propose_url(self):
        return api.portal.get_registry_record("smartweb.propose_events_url")

    @property
    def display_map(self):
        return self.context.display_map

    @property
    def only_past_events(self):
        return self.context.only_past_events

    @property
    def display_agendas_titles(self):
        return self.context.display_agendas_titles

    @property
    def show_categories_or_topics(self):
        return self.context.show_categories_or_topics



# -*- coding: utf-8 -*-

from imio.smartweb.core.contents.blocks.link.fields import IconsRadioWidget
from imio.smartweb.core.contents.blocks.link.fields import LinkField
from imio.smartweb.locales import SmartwebMessageFactory as _
from plone.app.contenttypes.interfaces import ILink
from plone.autoform import directives
from plone.dexterity.content import Container
from plone.namedfile.field import NamedBlobImage
from plone.supermodel import model
from zope import schema
from zope.interface import implementer


class IBlockLink(ILink):
    """Marker interface and Dexterity Python Schema for BlockLink"""

    title = schema.TextLine(title=_("Title"), required=True)

    description = schema.Text(title=_("Description"), required=False)

    open_in_new_tab = schema.Bool(
        title=_("Open in a new tab"), required=False, default=False
    )

    remoteUrl = LinkField(title=_("URL"), required=True)

    image = NamedBlobImage(title=_("Image"), required=False)

    model.fieldset("icons", label=_("Icons"), fields=["svg_icon"])
    directives.widget("svg_icon", IconsRadioWidget)
    svg_icon = schema.Choice(
        title=_("Link illustration icon"),
        description=_("Only used in table view (takes precedence over image)"),
        source="imio.smartweb.vocabulary.Icons",
        required=False,
    )


@implementer(IBlockLink)
class BlockLink(Container):
    """BlockLink class"""



# -*- coding: utf-8 -*-

from imio.smartweb.locales import SmartwebMessageFactory as _
from plone.autoform import directives
from plone.autoform.interfaces import IFormFieldProvider
from plone.supermodel import model
from zope import schema
from zope.interface import provider


@provider(IFormFieldProvider)
class IListing(model.Schema):
    directives.order_after(
        exclude_from_parent_listing="IExcludeFromNavigation.exclude_from_nav"
    )

    model.fieldset("settings", fields=["exclude_from_parent_listing"])
    exclude_from_parent_listing = schema.Bool(
        title=_("Exclude from parent listing"),
        description=_("If selected, this item will not appear in parent listing views"),
        required=False,
        default=False,
    )



# -*- coding: utf-8 -*-

from eea.facetednavigation.widgets import ViewPageTemplateFile
from eea.facetednavigation.widgets.select.widget import Widget


class Select(Widget):
    index = ViewPageTemplateFile("select.pt")



# -*- coding: utf-8 -*-

from imio.smartweb.core.contents import IRestView
from ZPublisher.BaseRequest import DefaultPublishTraverse
from zope.component import adapts
from zope.publisher.interfaces.http import IHTTPRequest


class RestViewTraversable(DefaultPublishTraverse):
    """ """

    adapts(IRestView, IHTTPRequest)

    def publishTraverse(self, request, name):
        if "u" in self.request.form and name != "view":
            return self.context

        return super(RestViewTraversable, self).publishTraverse(request, name)



# -*- coding: utf-8 -*-

from imio.smartweb.core.config import DIRECTORY_URL
from imio.smartweb.core.config import EVENTS_URL
from imio.smartweb.core.config import NEWS_URL
from imio.smartweb.core.contents.rest.search.endpoint import get_default_view_url
from imio.smartweb.core.utils import get_wca_token
from plone.protect.interfaces import IDisableCSRFProtection
from plone.restapi.deserializer import json_body
from plone.restapi.services import Service
from zope.interface import alsoProvides
from zope.interface import implementer
from zope.publisher.interfaces import IPublishTraverse

import os
import requests


@implementer(IPublishTraverse)
class BaseRequestForwarder(Service):
    def __init__(self, context, request):
        super().__init__(context, request)
        self.traversal_stack = []

    def reply(self):
        alsoProvides(self.request, IDisableCSRFProtection)
        url = "/".join(self.traversal_stack)
        auth_source_url = f"{self.base_url}/{url}"
        response = self.forward_request(auth_source_url)
        response = self.add_smartweb_urls(response)
        return response

    def publishTraverse(self, request, name):
        self.traversal_stack.append(name)
        return self

    def forward_request(self, url):
        method = self.request.method
        token = get_wca_token(self.client_id, self.client_secret)
        headers = {"Accept": "application/json", "Authorization": token}
        params = self.request.form
        if method == "GET":
            params = self.add_missing_metadatas(params)
        data = json_body(self.request)

        # Forward the request to the authentic source
        auth_source_response = requests.request(
            method, url, params=params, headers=headers, json=data
        )
        response = self.request.response
        # Set the status code and headers from the authentic source server response
        response.setStatus(auth_source_response.status_code)
        for header, value in auth_source_response.headers.items():
            response.setHeader(header, value)

        if auth_source_response.status_code == 204 or auth_source_response.text == "":
            # Empty response
            return ""

        return auth_source_response.json()

    def construct_url(self, view_url, item):
        # we can construct a Smartweb-related URL for item
        # TODO: handle other views & translations (use/refactor code in
        # search endpoint)
        item_uid = item["UID"]
        item_id = item.get("id", "content")
        item["smartweb_url"] = f"{view_url}/{item_id}?u={item_uid}"

    def add_smartweb_urls(self, json_data):
        if "items" not in json_data and "@id" not in json_data:
            return json_data
        default_view_url = get_default_view_url(self.request_type)
        if "@id" in json_data and "UID" in json_data:
            self.construct_url(default_view_url, json_data)
            return json_data
        for item in json_data.get("items", []):
            if "@id" in item and "UID" in item:
                self.construct_url(default_view_url, item)
        return json_data

    def add_missing_metadatas(self, params):
        if "fullobjects" in params:
            return params
        if "metadata_fields" not in params:
            params["metadata_fields"] = ["id", "UID"]
        else:
            if "id" not in params["metadata_fields"]:
                params["metadata_fields"].append("id")
            if "UID" not in params["metadata_fields"]:
                params["metadata_fields"].append("UID")
        return params


class DirectoryRequestForwarder(BaseRequestForwarder):
    request_type = "directory"
    client_id = os.environ.get("RESTAPI_DIRECTORY_CLIENT_ID")
    client_secret = os.environ.get("RESTAPI_DIRECTORY_CLIENT_SECRET")
    base_url = DIRECTORY_URL


class EventsRequestForwarder(BaseRequestForwarder):
    request_type = "events"
    client_id = os.environ.get("RESTAPI_EVENTS_CLIENT_ID")
    client_secret = os.environ.get("RESTAPI_EVENTS_CLIENT_SECRET")
    base_url = EVENTS_URL


class NewsRequestForwarder(BaseRequestForwarder):
    request_type = "news"
    client_id = os.environ.get("RESTAPI_NEWS_CLIENT_ID")
    client_secret = os.environ.get("RESTAPI_NEWS_CLIENT_SECRET")
    base_url = NEWS_URL



# -*- coding: utf-8 -*-

from plone.app.contenttypes.behaviors.viewlets import LeadImageViewlet


class NoLeadImageViewlet(LeadImageViewlet):
    """Leadimage viewlet that will be unavailable for pages & folders"""

    def update(self):
        super(NoLeadImageViewlet, self).update()
        self.available = False



# -*- coding: utf-8 -*-

from imio.smartweb.core.contents.sections.views import CarouselOrTableSectionView
from imio.smartweb.core.utils import batch_results
from imio.smartweb.core.utils import get_scale_url


class CollectionView(CarouselOrTableSectionView):
    """Collection Section view"""

    def items(self):
        max_items = self.context.nb_results_by_batch * self.context.max_nb_batches
        orientation = self.context.orientation
        image_scale = self.image_scale
        items = self.context.collection.to_object.results(
            batch=False, brains=True, limit=max_items
        )
        results = []
        for item in items:
            url = item.getURL()
            scale_url = get_scale_url(
                item, self.request, "image", image_scale, orientation
            )
            dict_item = {
                "title": item.Title,
                "description": item.Description,
                "effective": item.effective,
                "url": url,
                "has_image": item.has_leadimage,
            }
            if scale_url == "":
                dict_item["bad_scale"] = image_scale
                scale_url = f"{url}/@@images/image/{image_scale}"
            dict_item["image"] = scale_url
            results.append(dict_item)
        return batch_results(results, self.context.nb_results_by_batch)

    @property
    def see_all_url(self):
        return self.context.collection.to_object.absolute_url()



# -*- coding: utf-8 -*-

from imio.smartweb.core.contents.sections.base import ISection
from imio.smartweb.core.contents.sections.base import Section
from imio.smartweb.locales import SmartwebMessageFactory as _
from plone.supermodel import model
from z3c.relationfield.schema import RelationChoice
from z3c.relationfield.schema import RelationList
from zope import schema
from zope.interface import implementer


class ISectionSelections(ISection):
    """Marker interface and Dexterity Python Schema for SectionSelections"""

    selected_items = RelationList(
        title=_("Selected items"),
        value_type=RelationChoice(
            title="",
            vocabulary="plone.app.vocabularies.Catalog",
        ),
        required=True,
    )

    nb_results_by_batch = schema.Choice(
        title=_("Number of items per batch"),
        required=True,
        default=3,
        values=[1, 2, 3, 4],
    )

    model.fieldset("layout", fields=["show_items_lead_image", "show_items_description"])
    show_items_lead_image = schema.Bool(
        title=_("Show items lead image"), required=False
    )

    show_items_description = schema.Bool(
        title=_("Show items description"), required=False
    )


@implementer(ISectionSelections)
class SectionSelections(Section):
    """SectionSelections class"""

    manage_content = False
    manage_display = True



# -*- coding: utf-8 -*-

from imio.smartweb.core.contents.rest.view import BaseRestView
from imio.smartweb.core.interfaces import IOgpViewUtils
from plone import api
from zope.interface import implementer


@implementer(IOgpViewUtils)
class NewsViewView(BaseRestView):
    """NewsView view"""

    @property
    def propose_url(self):
        return api.portal.get_registry_record("smartweb.propose_news_url")

    @property
    def display_newsfolders_titles(self):
        return self.context.display_newsfolders_titles

    @property
    def show_categories_or_topics(self):
        return self.context.show_categories_or_topics



# -*- coding: utf-8 -*-

from embeddify import Embedder
from imio.smartweb.core.contents.sections.views import SectionView


class VideoView(SectionView):
    def get_embed_video(self, width=800, height=600):
        embedder = Embedder(width=width, height=height)
        return embedder(self.context.video_url, params=dict(autoplay=False))



# -*- coding: utf-8 -*-

from imio.smartweb.common.widgets.select import TranslatedAjaxSelectWidget
from imio.smartweb.core.contents.sections.base import ISection
from imio.smartweb.core.contents.sections.base import Section
from imio.smartweb.locales import SmartwebMessageFactory as _
from plone.app.z3cform.widget import RelatedItemsFieldWidget
from plone.app.z3cform.widget import SelectFieldWidget
from plone.autoform import directives
from plone.supermodel import model
from z3c.relationfield.schema import RelationChoice
from zope import schema
from zope.globalrequest import getRequest
from zope.i18n import translate
from zope.interface import implementer
from zope.interface import provider
from zope.schema.interfaces import IContextAwareDefaultFactory


@provider(IContextAwareDefaultFactory)
def see_all_default(context):
    return translate(_("See all news"), context=getRequest())


class ISectionNews(ISection):
    """Marker interface and Dexterity Python Schema for SectionNews"""

    directives.widget(related_news=SelectFieldWidget)
    related_news = schema.Choice(
        title=_("Related news folder"),
        source="imio.smartweb.vocabulary.RemoteNewsFolders",
        required=True,
    )

    specific_related_newsitems = schema.List(
        title=_("Specific related news"),
        description=_("Get priory on related news for these specifics news."),
        value_type=schema.Choice(source="imio.smartweb.vocabulary.NewsItemsFromEntity"),
        required=False,
    )
    directives.widget(
        "specific_related_newsitems",
        TranslatedAjaxSelectWidget,
        source="imio.smartweb.vocabulary.NewsItemsFromEntity",
    )

    directives.widget(
        "linking_rest_view",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "selectableTypes": ["imio.smartweb.NewsView"],
            "favorites": [],
        },
    )
    linking_rest_view = RelationChoice(
        title=_("News view used to display news items details"),
        vocabulary="plone.app.vocabularies.Catalog",
        required=True,
    )

    nb_results_by_batch = schema.Choice(
        title=_("Number of items per batch"),
        required=True,
        default=3,
        values=[1, 2, 3, 4],
    )

    max_nb_batches = schema.Int(
        title=_("Maximum number of batches to display"),
        required=True,
        default=2,
        min=1,
        max=12,
    )

    link_text = schema.TextLine(
        title=_("Text for the link to the news view"),
        defaultFactory=see_all_default,
        required=True,
    )

    model.fieldset("layout", fields=["show_items_description"])
    show_items_description = schema.Bool(
        title=_("Show items description"), required=False
    )

    model.fieldset("layout", fields=["display_newsfolders_titles"])
    display_newsfolders_titles = schema.Bool(
        title=_("Display news folders titles"),
        description=_("If checked, display news folders titles in the carousel view."),
        required=False,
    )


@implementer(ISectionNews)
class SectionNews(Section):
    """SectionNews class"""

    manage_display = True
    show_items_date = True



# -*- coding: utf-8 -*-

from imio.smartweb.core.browser.controlpanel import ISmartwebControlPanel
from imio.smartweb.core.contents import IPages
from eea.facetednavigation.interfaces import ICriteria
from eea.facetednavigation.subtypes.interfaces import IFacetedNavigable
from plone import api
from plone.app.imagecropping import PAI_STORAGE_KEY
from plone.registry.interfaces import IRegistry
from zope.annotation.interfaces import IAnnotations
from zope.component import getUtility
from zope.schema import getFieldNames

import logging

logger = logging.getLogger("imio.smartweb.core")
PROFILEID = "profile-imio.smartweb.core:default"


def configure_first_official_release(context):
    portal_setup = api.portal.get_tool("portal_setup")
    portal_setup.runImportStepFromProfile(PROFILEID, "plone.app.registry")
    portal_setup.runImportStepFromProfile(PROFILEID, "typeinfo")

    portal_setup.runImportStepFromProfile(PROFILEID, "catalog")
    portal_catalog = api.portal.get_tool("portal_catalog")
    # we could index only imio.smartweb.Page contents, but for some reason
    # api.content.find() doesn't find all objects
    portal_catalog.clearFindAndRebuild()
    logger.info("Reindexed catalog for category_and_topics index / metadata")


def set_thumb_scales(context):
    api.portal.set_registry_record("plone.thumb_scale_listing", "liste")
    api.portal.set_registry_record("plone.thumb_scale_summary", "vignette")


def update_actions(context):
    portal_setup = api.portal.get_tool("portal_setup")
    portal_setup.runImportStepFromProfile(PROFILEID, "actions")
    portal_actions = api.portal.get_tool("portal_actions")
    header_actions = getattr(portal_actions, "header_actions")
    if "e_guichet" in header_actions.objectIds():
        header_actions._delObject("e_guichet")
        logger.info("Deleted e_guichet header action")


def update_icons_and_names(context):
    portal_setup = api.portal.get_tool("portal_setup")
    portal_setup.runImportStepFromProfile(
        "profile-imio.smartweb.core:icons-basic", "plone.app.registry"
    )
    registry = getUtility(IRegistry)
    for old_icon in [
        "annuaire",
        "demarches",
        "ecoles",
        "horaires",
        "mobilite",
        "parkings",
        "sports",
        "tourisme",
        "travaux",
    ]:
        if f"smartweb.icon.{old_icon}" in registry:
            del registry.records[f"smartweb.icon.{old_icon}"]


def reload_registry(context):
    portal_setup = api.portal.get_tool("portal_setup")
    portal_setup.runImportStepFromProfile(PROFILEID, "plone.app.registry")


def reload_rolemap(context):
    portal_setup = api.portal.get_tool("portal_setup")
    portal_setup.runImportStepFromProfile(PROFILEID, "rolemap")


def reload_types(context):
    portal_setup = api.portal.get_tool("portal_setup")
    portal_setup.runImportStepFromProfile(PROFILEID, "typeinfo")


def reload_workflows(context):
    portal_setup = api.portal.get_tool("portal_setup")
    portal_setup.runImportStepFromProfile(PROFILEID, "workflow")


def check_itinerary_if_address_is_checked(context):
    brains = api.content.find(portal_type="imio.smartweb.SectionContact")
    for brain in brains:
        obj = brain.getObject()
        if "address" in obj.visible_blocks:
            if "itinerary" not in obj.visible_blocks:
                obj.visible_blocks.append("itinerary")
                obj._p_changed = 1


def add_placeholder_to_faceted_textsearch(context):
    brains = api.content.find(object_provides=IFacetedNavigable)
    for brain in brains:
        obj = brain.getObject()
        handler = ICriteria(obj)
        for criterion in handler.criteria:
            if criterion.widget == "text":
                criterion.placeholder = "Recherche"
                handler.criteria._p_changed = 1


def exclude_footers_from_parent_listing(context):
    brains = api.content.find(portal_type="imio.smartweb.Footer")
    for brain in brains:
        footer = brain.getObject()
        footer.exclude_from_parent_listing = True
        footer.reindexObject(idxs=["exclude_from_parent_listing"])


def reindex_all_pages(context):
    brains = api.content.find(object_provides=IPages)
    for brain in brains:
        try:
            obj = brain.getObject()
        except KeyError:
            logger.warn(f"getObject failed on {brain.getURL()}")
        else:
            obj.reindexObject()


def add_sendinblue_button_settings(context):
    fields = getFieldNames(ISmartwebControlPanel)
    fields.remove("sendinblue_button_position")
    fields.remove("sendinblue_button_text")
    registry = getUtility(IRegistry)
    registry.registerInterface(ISmartwebControlPanel, omit=fields, prefix="smartweb")


def find_multiple_categories_directory_views(context):
    brains = api.content.find(portal_type="imio.smartweb.DirectoryView")
    for brain in brains:
        directory = brain.getObject()
        if (
            directory.selected_categories is None
            or len(directory.selected_categories) <= 1
        ):
            continue
        msg = f"Found directory view with multiple categories : {directory.absolute_url()}"
        logger.warning(msg)
        api.portal.send_email(
            recipient="boulch@imio.be",
            subject="Multiple contacts categories in directory view",
            body=msg,
        )


def to_related_contacts(context):
    brains = api.content.find(portal_type="imio.smartweb.SectionContact")
    for brain in brains:
        obj = brain.getObject()
        if hasattr(obj, "related_contact"):
            setattr(obj, "related_contacts", [obj.related_contact])
            setattr(obj, "nb_contact_by_line", 1)
            delattr(obj, "related_contact")


def remove_deprecated_cropping_annotations(context):
    with api.env.adopt_user(username="admin"):
        brains = api.content.find(portal_type="imio.smartweb.Folder")
        for brain in brains:
            obj = brain.getObject()
            annotations = IAnnotations(obj)
            scales = annotations.get(PAI_STORAGE_KEY)
            if scales is not None and "banner_banner" in scales:
                del scales["banner_banner"]
                obj.reindexObject()
                logger.info(
                    f"Remove deprecated banner_banner cropping annotation on {obj.absolute_url()}"
                )


def migrate_is_in_portrait_mode(context):
    with api.env.adopt_user(username="admin"):
        brains = api.content.find(portal_type="imio.smartweb.SectionContact")
        for brain in brains:
            obj = brain.getObject()
            is_in_portrait_mode = getattr(obj, "is_in_portrait_mode", False)
            if is_in_portrait_mode:
                obj.orientation = "portrait"
                logger.info(
                    f"Migrated potrait mode to orientation portrait for {obj.absolute_url()}"
                )


def migrate_old_scales_from_vocabulary(context):
    with api.env.adopt_user(username="admin"):
        brains = api.content.find(
            portal_type=[
                "imio.smartweb.SectionContact",
                "imio.smartweb.SectionGallery",
            ]
        )
        for brain in brains:
            obj = brain.getObject()
            old_scale = obj.image_scale
            if old_scale in ["affiche", "vignette", "liste"]:
                continue
            new_scale = "affiche"
            if old_scale == "preview":
                new_scale = "vignette"
            obj.image_scale = new_scale
            logger.info(
                f"Migrated deprecated scale from {old_scale} to {new_scale} for {obj.absolute_url()}"
            )


def migrate_old_sizes_from_section_text(context):
    with api.env.adopt_user(username="admin"):
        brains = api.content.find(portal_type=["imio.smartweb.SectionText"])
        for brain in brains:
            obj = brain.getObject()
            old_scale = obj.image_size
            if old_scale in ["affiche", "vignette"]:
                continue
            new_scale = "affiche"
            if old_scale == "preview":
                new_scale = "vignette"
            obj.image_size = new_scale
            logger.info(
                f"Migrated deprecated scale from {old_scale} to {new_scale} for {obj.absolute_url()}"
            )



# -*- coding: utf-8 -*-

from imio.smartweb.core.contents import IFolder
from imio.smartweb.core.interfaces import IImioSmartwebCoreLayer
from imio.smartweb.core.testing import IMIO_SMARTWEB_CORE_FUNCTIONAL_TESTING
from imio.smartweb.core.testing import ImioSmartwebTestCase
from imio.smartweb.core.tests.utils import make_named_image
from plone import api
from plone.api.exc import InvalidParameterError
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import TEST_USER_PASSWORD
from plone.app.testing import setRoles
from plone.app.textfield.value import RichTextValue
from plone.uuid.interfaces import IUUID
from plone.dexterity.interfaces import IDexterityFTI
from plone.namedfile.file import NamedBlobImage
from plone.testing.zope import Browser
from time import sleep
from zope.annotation.interfaces import IAnnotations
from zope.component import createObject
from zope.component import getMultiAdapter
from zope.component import queryUtility
from zope.interface import alsoProvides
from zope.lifecycleevent import modified

import json
import transaction


class TestFolder(ImioSmartwebTestCase):
    layer = IMIO_SMARTWEB_CORE_FUNCTIONAL_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.authorized_types_in_folder = [
            "Collection",
            "Link",
            "imio.smartweb.Folder",
            "imio.smartweb.Page",
            "imio.smartweb.PortalPage",
            "imio.smartweb.Procedure",
        ]

        self.unauthorized_types_in_folder = ["File", "Image", "Document"]

        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

    def test_ct_folder_schema(self):
        fti = queryUtility(IDexterityFTI, name="imio.smartweb.Folder")
        schema = fti.lookupSchema()
        self.assertEqual(IFolder, schema)

    def test_ct_folder_fti(self):
        fti = queryUtility(IDexterityFTI, name="imio.smartweb.Folder")
        self.assertTrue(fti)

    def test_ct_folder_factory(self):
        fti = queryUtility(IDexterityFTI, name="imio.smartweb.Folder")
        factory = fti.factory
        obj = createObject(factory)

        self.assertTrue(
            IFolder.providedBy(obj),
            "IFolder not provided by {0}!".format(
                obj,
            ),
        )

    def test_ct_folder_adding(self):
        setRoles(self.portal, TEST_USER_ID, ["Contributor"])
        obj = api.content.create(
            container=self.portal,
            type="imio.smartweb.Folder",
            id="folder",
        )

        self.assertTrue(
            IFolder.providedBy(obj),
            "IFolder not provided by {0}!".format(
                obj.id,
            ),
        )

        parent = obj.__parent__
        self.assertIn("folder", parent.objectIds())

        # check that deleting the object works too
        api.content.delete(obj=obj)
        self.assertNotIn("folder", parent.objectIds())

    def test_ct_folder_globally_addable(self):
        setRoles(self.portal, TEST_USER_ID, ["Contributor"])
        fti = queryUtility(IDexterityFTI, name="imio.smartweb.Folder")
        self.assertTrue(fti.global_allow, "{0} is not globally addable!".format(fti.id))

    def test_ct_folder_filter_content_type_true(self):
        setRoles(self.portal, TEST_USER_ID, ["Contributor"])
        fti = queryUtility(IDexterityFTI, name="imio.smartweb.Folder")
        portal_types = self.portal.portal_types
        parent_id = portal_types.constructContent(
            fti.id,
            self.portal,
            "folder_id",
            title="Folder container",
        )
        folder = self.portal[parent_id]
        for t in self.unauthorized_types_in_folder:
            with self.assertRaises(InvalidParameterError):
                api.content.create(
                    container=folder,
                    type=t,
                    title="My {}".format(t),
                )
        for t in self.authorized_types_in_folder:
            api.content.create(
                container=folder,
                type=t,
                title="My {}".format(t),
            )

    def test_leadimage_in_folder_block_view(self):
        folder = api.content.create(
            container=self.portal,
            type="imio.smartweb.Folder",
            title="My folder",
        )
        page = api.content.create(
            container=folder,
            type="imio.smartweb.Page",
            title="My page",
        )
        page.image = NamedBlobImage(**make_named_image())
        alsoProvides(self.request, IImioSmartwebCoreLayer)
        view = getMultiAdapter((folder, self.request), name="block_view")
        self.assertNotIn("newsImage", view())

        view = getMultiAdapter((folder, self.request), name="block_view_with_images")
        self.assertIn("newsImage", view())
        self.assertEqual(view.get_thumb_scale_summary(), "paysage_vignette")

        self.assertIn("display-paysage", view())
        folder.orientation = "portrait"
        self.assertIn("display-portrait", view())

        api.portal.set_registry_record("plone.thumb_scale_summary", "preview")
        annotations = IAnnotations(self.request)
        del annotations["plone.memoize"]
        view = getMultiAdapter((folder, self.request), name="block_view_with_images")
        self.assertEqual(view.get_thumb_scale_summary(), "preview")

        api.portal.set_registry_record("plone.no_thumbs_summary", True)
        annotations = IAnnotations(self.request)
        del annotations["plone.memoize"]
        view = getMultiAdapter((folder, self.request), name="block_view_with_images")
        self.assertIsNone(view.get_thumb_scale_summary())

    def test_element_view_as_anonymous(self):
        folder = api.content.create(
            container=self.portal,
            type="imio.smartweb.Folder",
            title="Folder",
            id="folder",
        )
        api.content.transition(folder, "publish")
        page1 = api.content.create(
            container=folder,
            type="imio.smartweb.Page",
            title="Page 1",
            id="page1",
        )
        api.content.transition(page1, "publish")
        page2 = api.content.create(
            container=folder,
            type="imio.smartweb.Page",
            title="Page 2",
            id="page2",
        )
        api.content.transition(page2, "publish")
        folder.setLayout("element_view")
        transaction.commit()

        # Anonymous - no default page set on element view
        browser = Browser(self.layer["app"])
        browser.open(folder.absolute_url())
        content = browser.contents
        self.assertIn("template-summary_view", content)
        self.assertNotIn("template-full_view", content)
        self.assertEqual(content.count("<h2"), 2)

        # Anonymous - default page is set on element view
        folder.set_default_item(old_default_item=None, new_default_item=page1)
        transaction.commit()
        browser.open(folder.absolute_url())
        content = browser.contents
        self.assertIn("template-full_view", content)
        self.assertNotIn("template-summary_view", content)
        self.assertIn("<h1>Page 1</h1>", content)

    def test_element_view_as_editor(self):
        folder = api.content.create(
            container=self.portal,
            type="imio.smartweb.Folder",
            title="Folder",
            id="folder",
        )
        page1 = api.content.create(
            container=folder,
            type="imio.smartweb.Page",
            title="Page 1",
            id="page1",
        )
        uuid = IUUID(page1)
        api.content.create(
            container=folder,
            type="imio.smartweb.Page",
            title="Page 2",
            id="page2",
        )
        folder.setLayout("element_view")
        transaction.commit()

        browser = Browser(self.layer["app"])
        browser.addHeader(
            "Authorization",
            "Basic %s:%s"
            % (
                TEST_USER_NAME,
                TEST_USER_PASSWORD,
            ),
        )
        browser.open(folder.absolute_url())
        content = browser.contents
        self.assertIn(
            '<h1 class="documentFirstHeading">Form to choose item to be displayed as the home page of the folder</h1>',
            content,
        )
        radio_control = browser.getControl(name="form.widgets.default_page_uid")
        self.assertIn(uuid, radio_control.options)
        self.assertEqual(radio_control.value, [])
        radio_control.value = uuid
        browser.getControl(name="form.buttons.apply").click()
        content = browser.contents
        self.assertIn("Data successfully updated.", content)
        radio_control = browser.getControl(name="form.widgets.default_page_uid")
        self.assertEqual(radio_control.value, [uuid])

    def test_default_page_in_folder_contents(self):
        folder = api.content.create(
            container=self.portal,
            type="imio.smartweb.Folder",
            id="folder",
        )
        page1 = api.content.create(
            container=folder, type="imio.smartweb.Page", id="page1"
        )
        context_info_view = getMultiAdapter(
            (folder, self.request), name="fc-contextInfo"
        )
        result = json.loads(context_info_view())
        self.assertIsNone(result["defaultPage"])

        folder.setLayout("element_view")
        folder.set_default_item(new_default_item=page1)

        result = json.loads(context_info_view())
        self.assertEqual(result["defaultPage"], "page1")

    def test_modification_date(self):
        folder = api.content.create(
            container=self.portal,
            type="imio.smartweb.Folder",
            id="folder",
        )
        first_modification = folder.ModificationDate()
        sleep(1)
        page = api.content.create(
            container=folder,
            type="imio.smartweb.Page",
            title="Page",
        )
        next_modification = folder.ModificationDate()
        self.assertNotEqual(first_modification, next_modification)

        first_modification = folder.ModificationDate()
        sleep(1)
        page.title = "New Page"
        modified(page)
        page.reindexObject()
        next_modification = folder.ModificationDate()
        self.assertNotEqual(first_modification, next_modification)

        first_modification = folder.ModificationDate()
        sleep(1)
        section = api.content.create(
            container=page,
            type="imio.smartweb.SectionText",
            title="Section text",
        )
        modified(section)
        next_modification = folder.ModificationDate()
        self.assertNotEqual(first_modification, next_modification)

        section = api.content.create(
            container=page,
            type="imio.smartweb.SectionGallery",
            title="Section Gallery",
        )
        first_modification = folder.ModificationDate()
        sleep(1)
        api.content.create(
            container=section,
            type="Image",
            title="Image",
        )
        next_modification = folder.ModificationDate()
        self.assertEqual(first_modification, next_modification)

    def test_remove_folder_and_children(self):
        folder = api.content.create(
            container=self.portal,
            type="imio.smartweb.Folder",
            id="folder",
        )
        subfolder = api.content.create(
            container=folder,
            type="imio.smartweb.Folder",
            id="subfolder",
        )
        page1 = api.content.create(
            container=folder,
            type="imio.smartweb.Page",
            id="page1",
        )
        page2 = api.content.create(
            container=subfolder,
            type="imio.smartweb.Page",
            id="page1",
        )
        section_text = api.content.create(
            container=page1,
            type="imio.smartweb.SectionText",
            id="txt",
        )
        api.content.create(
            container=page2,
            type="imio.smartweb.SectionText",
            id="txt",
        )
        section_text.text = RichTextValue(
            "<p>Textsectionbody</p>", "text/html", "text/html"
        )
        section_text.reindexObject()
        api.content.delete(folder)

        uuid = IUUID(folder)
        brains = api.content.find(UID=uuid)
        self.assertEqual(len(brains), 0)

        uuid = IUUID(subfolder)
        brains = api.content.find(UID=uuid)
        self.assertEqual(len(brains), 0)

        uuid = IUUID(page1)
        brains = api.content.find(UID=uuid)
        self.assertEqual(len(brains), 0)

        uuid = IUUID(page2)
        brains = api.content.find(UID=uuid)
        self.assertEqual(len(brains), 0)

    def test_select_view_template(self):
        folder = api.content.create(
            container=self.portal,
            type="imio.smartweb.Folder",
            title="My folder",
        )
        self.request.form = {
            "templateId": "block_view_with_images",
            "form.buttons.apply": "Apply",
        }
        creation_date = folder.ModificationDate()
        view = getMultiAdapter((folder, self.request), name="selectViewTemplate")
        sleep(1)
        view.selectViewTemplate()
        new_date = folder.ModificationDate()
        self.assertNotEqual(creation_date, new_date)



# -*- coding: utf-8 -*-

from imio.smartweb.locales import SmartwebMessageFactory as _
from plone import api
from plone.autoform.directives import write_permission
from plone.dexterity.content import Container
from plone.namedfile.field import NamedBlobImage
from plone.supermodel import model
from zope import schema
from zope.i18n import translate
from zope.interface import implementer


class ISection(model.Schema):
    """Shared base marker interface and schema for Sections"""

    title = schema.TextLine(title=_("Title"), required=True)

    model.fieldset(
        "layout",
        label=_("Layout"),
        fields=[
            "hide_title",
            "collapsible_section",
            "background_image",
            "bootstrap_css_class",
            "css_class",
        ],
    )

    hide_title = schema.Bool(
        title=_("Hide title"),
        description=_(
            "Hide title for anonymous user. Title stays visible for the editor."
        ),
        required=False,
        default=False,
    )

    collapsible_section = schema.Bool(
        title=_(
            "Hide the content of the section that will be displayed by clicking on the title"
        ),
        description=_(
            "The title will always be displayed if this behavior is enabled."
        ),
        required=False,
        default=False,
    )

    write_permission(background_image="cmf.ManagePortal")
    background_image = NamedBlobImage(
        title=_("Background image"),
        required=False,
    )

    bootstrap_css_class = schema.Choice(
        title=_("Section width"),
        required=False,
        vocabulary="imio.smartweb.vocabulary.BootstrapCSS",
    )

    write_permission(css_class="cmf.ManagePortal")
    css_class = schema.TextLine(title=_("CSS class"), required=False)


@implementer(ISection)
class Section(Container):
    """Shared base class for Sections"""

    manage_content = False
    manage_display = False
    can_toggle_title_visibility = True

    def canSetLayout(self):
        return False

    def canSetDefaultPage(self):
        return False

    def section_error(self, *args, **kwargs):
        if api.user.is_anonymous():
            return ""
        return translate(
            _(
                '<div class="section_error_txt">Error in section : "{}" <a href="{}/edit">[edit]</a></div>'
            ),
            target_language=api.portal.get_current_language()[:2],
        ).format(self.title, self.absolute_url())



# -*- coding: utf-8 -*-

from imio.smartweb.locales import SmartwebMessageFactory as _
from plone.autoform import directives
from plone.autoform.interfaces import IFormFieldProvider
from plone.supermodel import model
from zope import schema
from zope.interface import provider


@provider(IFormFieldProvider)
class ICategoryDisplay(model.Schema):
    directives.order_before(show_categories_or_topics="IOrientation.orientation")

    model.fieldset("layout", fields=["show_categories_or_topics"])
    show_categories_or_topics = schema.Choice(
        title=_("Show category or topic"),
        description=_(
            "Select if you want (specific) category or (first) topic displayed on items"
        ),
        source="imio.smartweb.vocabulary.CategoriesDisplay",
        required=True,
        default="category",
    )



# -*- coding: utf-8 -*-

from imio.smartweb.core.testing import IMIO_SMARTWEB_CORE_FUNCTIONAL_TESTING
from imio.smartweb.core.testing import ImioSmartwebTestCase
from imio.smartweb.core.tests.utils import make_named_image
from imio.smartweb.core.viewlets.banner import BannerViewlet
from plone import api
from plone.app.testing import logout
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.namedfile.file import NamedBlobImage
from zope.component import getMultiAdapter


class TestBanner(ImioSmartwebTestCase):
    layer = IMIO_SMARTWEB_CORE_FUNCTIONAL_TESTING

    def setUp(self):
        """Custom shared utility setup for tests"""
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.folder = api.content.create(
            container=self.portal,
            type="imio.smartweb.Folder",
            title="Folder",
            id="folder",
        )

    def test_viewlet_banner(self):
        viewlet = BannerViewlet(self.folder, self.request, None, None)
        viewlet.update()
        self.assertFalse(viewlet.available())
        self.folder.banner = NamedBlobImage(**make_named_image())
        self.assertTrue(viewlet.available())
        self.assertIn(
            "background-image:url(http://nohost/plone/folder/@@images/banner?cache_key=",
            viewlet.background_style(),
        )
        template = self.folder.restrictedTraverse("view")
        view = self.folder.restrictedTraverse("@@plone_layout")
        body_class = view.bodyClass(template, view)
        self.assertIn("bloc-banner", body_class)

    def test_no_banner(self):
        viewlet = BannerViewlet(self.folder, self.request, None, None)
        viewlet.update()
        self.assertFalse(viewlet.available())
        template = self.folder.restrictedTraverse("view")
        view = self.folder.restrictedTraverse("@@plone_layout")
        body_class = view.bodyClass(template, view)
        self.assertNotIn("bloc-banner", body_class)

    def test_banner_root_locally_hidden(self):
        self.folder.banner = NamedBlobImage(**make_named_image())
        viewlet = BannerViewlet(self.folder, self.request, None, None)
        viewlet.update()
        self.assertTrue(viewlet.available())
        self.assertFalse(viewlet.is_banner_locally_hidden)
        switch_banner_display = getMultiAdapter(
            (self.folder, self.request), name="switch_banner_display"
        )
        switch_banner_display()
        self.assertTrue(viewlet.is_banner_locally_hidden)
        template = self.folder.restrictedTraverse("view")
        view = self.folder.restrictedTraverse("@@plone_layout")
        body_class = view.bodyClass(template, view)
        self.assertNotIn("bloc-banner", body_class)
        subfolder = api.content.create(
            container=self.folder,
            type="imio.smartweb.Folder",
            title="Subfolder",
        )
        subfolder_viewlet = BannerViewlet(subfolder, self.request, None, None)
        subfolder_viewlet.update()
        self.assertFalse(subfolder_viewlet.available())
        self.assertFalse(subfolder_viewlet.is_banner_locally_hidden)
        self.assertEqual(subfolder_viewlet.background_style(), "")
        template = subfolder.restrictedTraverse("view")
        view = subfolder.restrictedTraverse("@@plone_layout")
        body_class = view.bodyClass(template, view)
        self.assertNotIn("bloc-banner", body_class)

    def test_banner_child_locally_hidden(self):
        self.folder.banner = NamedBlobImage(**make_named_image())
        viewlet = BannerViewlet(self.folder, self.request, None, None)
        viewlet.update()
        self.assertTrue(viewlet.available())
        subfolder = api.content.create(
            container=self.folder,
            type="imio.smartweb.Folder",
            title="Subfolder",
        )
        page = api.content.create(
            container=subfolder,
            type="imio.smartweb.Page",
            title="Page",
        )

        switch_banner_display = getMultiAdapter(
            (subfolder, self.request), name="switch_banner_display"
        )
        # Hide banner on subfolder
        switch_banner_display()
        subfolder_viewlet = BannerViewlet(subfolder, self.request, None, None)
        subfolder_viewlet.update()
        self.assertTrue(subfolder_viewlet.available())
        self.assertTrue(subfolder_viewlet.is_banner_locally_hidden)
        self.assertEqual(subfolder_viewlet.background_style(), "")
        page_viewlet = BannerViewlet(page, self.request, None, None)
        page_viewlet.update()
        self.assertFalse(page_viewlet.available())
        self.assertFalse(page_viewlet.is_banner_locally_hidden)
        self.assertEqual(page_viewlet.background_style(), "")

        # Show banner again on subfolder
        switch_banner_display()
        subfolder_viewlet = BannerViewlet(subfolder, self.request, None, None)
        subfolder_viewlet.update()
        self.assertTrue(subfolder_viewlet.available())
        self.assertFalse(subfolder_viewlet.is_banner_locally_hidden)
        # images_view = queryMultiAdapter((self.folder, self.request), name="images")
        # scale = images_view.scale("banner", "banner")
        # css_bg_image = f"background-image:url({scale.url});"
        # css_bg_size = "background-size:cover;"
        self.assertIn(
            "background-image:url(http://nohost/plone/folder/@@images/banner?cache_key=",
            subfolder_viewlet.background_style(),
        )
        page_viewlet = BannerViewlet(page, self.request, None, None)
        page_viewlet.update()
        self.assertTrue(page_viewlet.available())
        self.assertFalse(page_viewlet.is_banner_locally_hidden)
        self.assertIn(
            "background-image:url(http://nohost/plone/folder/@@images/banner?cache_key=",
            page_viewlet.background_style(),
        )

    def test_banner_can_edit_locally_hidden(self):
        self.folder.banner = NamedBlobImage(**make_named_image())
        viewlet = BannerViewlet(self.folder, self.request, None, None)
        viewlet.update()
        self.assertTrue(viewlet.available())
        subfolder = api.content.create(
            container=self.folder,
            type="imio.smartweb.Folder",
            title="Subfolder",
        )

        switch_banner_display = getMultiAdapter(
            (subfolder, self.request), name="switch_banner_display"
        )
        # Hide banner on subfolder
        switch_banner_display()
        subfolder_viewlet = BannerViewlet(subfolder, self.request, None, None)
        subfolder_viewlet.update()
        self.assertTrue(subfolder_viewlet.available())
        self.assertTrue(subfolder_viewlet.is_banner_locally_hidden)
        self.assertEqual(subfolder_viewlet.background_style(), "")
        template = subfolder.restrictedTraverse("view")
        view = subfolder.restrictedTraverse("@@plone_layout")
        body_class = view.bodyClass(template, view)
        self.assertNotIn("bloc-banner", body_class)
        logout()
        self.assertFalse(subfolder_viewlet.available())

    def test_banner_on_minisite(self):
        self.folder.banner = NamedBlobImage(**make_named_image())
        view = getMultiAdapter((self.folder, self.request), name="minisite_settings")
        view.enable()
        viewlet = BannerViewlet(self.folder, self.request, None, None)
        viewlet.update()
        self.assertTrue(viewlet.available())

        subfolder = api.content.create(
            container=self.folder,
            type="imio.smartweb.Folder",
            title="Subfolder",
        )
        viewlet = BannerViewlet(subfolder, self.request, None, None)
        viewlet.update()
        self.assertTrue(viewlet.available())



# -*- coding: utf-8 -*-

from imio.smartweb.core.contents import IPages
from imio.smartweb.core.contents import Pages
from imio.smartweb.locales import SmartwebMessageFactory as _
from zope.interface import implementer
from zope import schema


class ICirkwiView(IPages):
    """Marker interface and Dexterity Python Schema for CirkwiView"""

    cirkwi_widget_id = schema.TextLine(
        title=_("Cirkwi widget ID"),
        description=_(
            "Specify your cirkwi widget ID. You can find it when you create a new widget in Cirkwi."
        ),
        required=True,
    )

    cirkwi_widget_mb_key = schema.TextLine(
        title=_("Cirkwi widget mb_key"),
        required=True,
    )


@implementer(ICirkwiView)
class CirkwiView(Pages):
    """CirkwiView class"""



# -*- coding: utf-8 -*-

from imio.smartweb.locales import SmartwebMessageFactory as _
from plone.app.z3cform.widget import RelatedItemsFieldWidget
from plone.autoform import directives
from plone.autoform.interfaces import IFormFieldProvider
from plone.supermodel import model
from z3c.relationfield.schema import RelationChoice
from z3c.relationfield.schema import RelationList
from zope.interface import provider


@provider(IFormFieldProvider)
class IQuickAccessSelection(model.Schema):
    quick_access_items = RelationList(
        title=_("Quick access contents"),
        value_type=RelationChoice(
            title="Items selection",
            vocabulary="plone.app.vocabularies.Catalog",
        ),
        required=False,
    )
    directives.widget(
        "quick_access_items",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "favorites": [],
        },
    )

    model.fieldset(
        "categorization", label=_("Categorization"), fields=["quick_access_items"]
    )



# -*- coding: utf-8 -*-

from plone.app.layout.viewlets import common
from zope.viewlet.interfaces import IViewletManager


class PortalHeaderTopContainerViewlet(common.ViewletBase):
    """Viewlet containing the top header viewlets manager"""


class IPortalHeaderTopContainerViewletManager(IViewletManager):
    """Viewlet manager containing top header viewlets"""



# -*- coding: utf-8 -*-

from imio.smartweb.locales import SmartwebMessageFactory as _
from plone import api
from Products.Five.browser import BrowserView
from zope.interface import Interface
from zope.interface import alsoProvides
from zope.interface import noLongerProvides


class ILocallyHiddenBanner(Interface):
    """ """


class BannerSettings(BrowserView):
    """Banner settings"""

    def switch_banner_display(self):
        if ILocallyHiddenBanner.providedBy(self.context):
            noLongerProvides(self.context, ILocallyHiddenBanner)
            message = _("Banner is now again displayed from this item")
        else:
            alsoProvides(self.context, ILocallyHiddenBanner)
            message = _("Banner is now hidden from this item")
        api.portal.show_message(message, self.request)
        self.context.reindexObject(idxs=["object_provides"])
        self.request.response.redirect(self.context.absolute_url())



# -*- coding: utf-8 -*-

from imio.smartweb.core.contents import RestView
from imio.smartweb.locales import SmartwebMessageFactory as _
from plone.autoform import directives
from plone.supermodel import model
from z3c.form.browser.checkbox import CheckBoxFieldWidget
from zope import schema
from zope.interface import implementer


class IEventsView(model.Schema):
    """Marker interface and Dexterity Python Schema for EventsView"""

    directives.widget(selected_event_types=CheckBoxFieldWidget)
    selected_event_types = schema.List(
        title=_("Selected event types"),
        value_type=schema.Choice(vocabulary="imio.smartweb.vocabulary.EventsTypes"),
        required=True,
    )

    selected_agenda = schema.Choice(
        title=_("Selected agenda"),
        description=_("Select agenda to display"),
        vocabulary="imio.smartweb.vocabulary.RemoteAgendas",
        required=True,
    )

    nb_results = schema.Int(
        title=_("Number of items to display"), default=20, required=True
    )

    display_map = schema.Bool(
        title=_("Display map"),
        description=_("If selected, map will be displayed"),
        required=False,
        default=True,
    )

    display_agendas_titles = schema.Bool(
        title=_("Display agendas titles"),
        description=_("If checked, display agendas titles"),
        required=False,
    )

    only_past_events = schema.Bool(
        title=_("Display only past events"),
        description=_("If selected, only past events will be displayed"),
        required=False,
        default=False,
    )


@implementer(IEventsView)
class EventsView(RestView):
    """EventsView class"""



# -*- coding: utf-8 -*-

from collective.geolocationbehavior.geolocation import IGeolocatable
from imio.smartweb.core.contents import IPage
from imio.smartweb.core.interfaces import IImioSmartwebCoreLayer
from imio.smartweb.core.testing import IMIO_SMARTWEB_CORE_INTEGRATION_TESTING
from imio.smartweb.core.testing import ImioSmartwebTestCase
from plone import api
from plone.api.exc import InvalidParameterError
from plone.app.testing import logout
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.dexterity.interfaces import IDexterityFTI
from plone.formwidget.geolocation.geolocation import Geolocation
from plone.uuid.interfaces import IUUID
from time import sleep
from unittest.mock import patch
from zope.component import createObject
from zope.component import getMultiAdapter
from zope.component import queryUtility
from zope.interface import alsoProvides
from zope.lifecycleevent import modified


class TestPage(ImioSmartwebTestCase):
    layer = IMIO_SMARTWEB_CORE_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests"""
        self.authorized_types_in_page = [
            "imio.smartweb.SectionContact",
            "imio.smartweb.SectionFiles",
            "imio.smartweb.SectionGallery",
            "imio.smartweb.SectionLinks",
            "imio.smartweb.SectionSelections",
            "imio.smartweb.SectionText",
            "imio.smartweb.SectionVideo",
        ]
        self.unauthorized_types_in_page = [
            "Document",
            "Link",
            "File",
            "Image",
            "imio.smartweb.BlockLink",
            "imio.smartweb.DirectoryView",
            "imio.smartweb.EventsView",
            "imio.smartweb.Folder",
            "imio.smartweb.Footer",
            "imio.smartweb.NewsView",
            "imio.smartweb.Page",
            "imio.smartweb.PortalPage",
            "imio.smartweb.Procedure",
        ]

        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.folder = api.content.create(
            container=self.portal,
            type="imio.smartweb.Folder",
            id="folder",
        )

    def test_ct_page_schema(self):
        fti = queryUtility(IDexterityFTI, name="imio.smartweb.Page")
        schema = fti.lookupSchema()
        self.assertEqual(IPage, schema)

    def test_ct_page_fti(self):
        fti = queryUtility(IDexterityFTI, name="imio.smartweb.Page")
        self.assertTrue(fti)

    def test_ct_page_factory(self):
        fti = queryUtility(IDexterityFTI, name="imio.smartweb.Page")
        factory = fti.factory
        obj = createObject(factory)

        self.assertTrue(
            IPage.providedBy(obj),
            "IPage not provided by {0}!".format(
                obj,
            ),
        )

    def test_ct_page_adding(self):
        setRoles(self.portal, TEST_USER_ID, ["Contributor"])
        page = api.content.create(
            container=self.folder,
            type="imio.smartweb.Page",
            title="page",
        )
        self.assertTrue(
            IPage.providedBy(page),
            "IPage not provided by {0}!".format(
                page.id,
            ),
        )
        parent = page.__parent__
        self.assertIn("page", parent.objectIds())

        # check that deleting the object works too
        api.content.delete(obj=page)
        self.assertNotIn("page", parent.objectIds())

    def test_ct_page_globally_addable(self):
        setRoles(self.portal, TEST_USER_ID, ["Contributor"])
        fti = queryUtility(IDexterityFTI, name="imio.smartweb.Page")
        self.assertTrue(fti.global_allow, "{0} is not globally addable!".format(fti.id))

    def test_ct_page_filter_content_type_true(self):
        setRoles(self.portal, TEST_USER_ID, ["Contributor"])
        page = api.content.create(
            container=self.folder,
            type="imio.smartweb.Page",
            title="page",
        )
        for t in self.unauthorized_types_in_page:
            with self.assertRaises(InvalidParameterError):
                api.content.create(
                    container=page,
                    type=t,
                    title="My {}".format(t),
                )
        for t in self.authorized_types_in_page:
            api.content.create(
                container=page,
                type=t,
                title="My {}".format(t),
            )

    def test_js_bundles(self):
        page = api.content.create(
            container=self.portal,
            type="imio.smartweb.Page",
            title="Page",
        )
        alsoProvides(self.request, IImioSmartwebCoreLayer)

        getMultiAdapter((page, self.request), name="full_view")()
        bundles = getattr(self.request, "enabled_bundles", [])
        self.assertEqual(len(bundles), 0)

        api.content.create(
            container=page,
            type="imio.smartweb.SectionGallery",
            title="Gallery",
        )
        getMultiAdapter((page, self.request), name="full_view")()
        bundles = getattr(self.request, "enabled_bundles", [])
        self.assertEqual(len(bundles), 2)
        self.assertListEqual(bundles, ["spotlight", "flexbin"])

    def test_js_bundles_with_contact(self):
        page = api.content.create(
            container=self.portal,
            type="imio.smartweb.Page",
            title="Page",
        )
        alsoProvides(self.request, IImioSmartwebCoreLayer)

        getMultiAdapter((page, self.request), name="full_view")()
        bundles = getattr(self.request, "enabled_bundles", [])
        self.assertEqual(len(bundles), 0)

        section_contact = api.content.create(
            container=page,
            type="imio.smartweb.SectionContact",
            title="Contact",
        )
        getMultiAdapter((page, self.request), name="full_view")()
        bundles = getattr(self.request, "enabled_bundles", [])
        self.assertEqual(len(bundles), 2)
        self.assertListEqual(bundles, ["spotlight", "flexbin"])

        section_contact.gallery_mode = "swiper"
        setattr(self.request, "enabled_bundles", [])
        getMultiAdapter((page, self.request), name="full_view")()
        bundles = getattr(self.request, "enabled_bundles", [])
        self.assertEqual(len(bundles), 1)
        self.assertListEqual(bundles, ["spotlight"])

    def test_modification_date(self):
        page = api.content.create(
            container=self.folder,
            type="imio.smartweb.Page",
            title="Page",
        )
        section = api.content.create(
            container=page,
            type="imio.smartweb.SectionText",
            title="Section text",
        )
        first_modification = page.ModificationDate()
        sleep(1)
        modified(section)
        next_modification = page.ModificationDate()
        self.assertNotEqual(first_modification, next_modification)

        section = api.content.create(
            container=page,
            type="imio.smartweb.SectionGallery",
            title="Section Gallery",
        )
        first_modification = page.ModificationDate()
        sleep(1)
        api.content.create(
            container=section,
            type="Image",
            title="Image",
        )
        next_modification = page.ModificationDate()
        self.assertNotEqual(first_modification, next_modification)

        section = api.content.create(
            container=page,
            type="imio.smartweb.SectionLinks",
            title="Section Links",
        )
        link = api.content.create(
            container=section,
            type="imio.smartweb.BlockLink",
            title="Link 1",
        )
        first_modification = page.ModificationDate()
        sleep(1)
        link.title = "Link 1 - new"
        modified(link)
        next_modification = page.ModificationDate()
        self.assertNotEqual(first_modification, next_modification)

    def test_geolocation(self):
        catalog = api.portal.get_tool("portal_catalog")
        page = api.content.create(
            container=self.folder,
            type="imio.smartweb.Page",
            title="Page",
        )
        uuid = IUUID(page)
        brain = api.content.find(UID=uuid)[0]
        indexes = catalog.getIndexDataForRID(brain.getRID())
        self.assertEqual(indexes.get("latitude"), "")
        self.assertEqual(indexes.get("longitude"), "")

        section = api.content.create(
            container=page,
            type="imio.smartweb.SectionMap",
            title="Section Map",
        )
        brain = api.content.find(UID=uuid)[0]
        indexes = catalog.getIndexDataForRID(brain.getRID())
        self.assertEqual(indexes.get("latitude"), "")
        self.assertEqual(indexes.get("longitude"), "")

        section2 = api.content.create(
            container=page,
            type="imio.smartweb.SectionMap",
            title="Section Map 2",
        )
        IGeolocatable(section2).geolocation = Geolocation(
            latitude="4.5", longitude="45"
        )
        modified(section2)
        brain = api.content.find(UID=uuid)[0]
        indexes = catalog.getIndexDataForRID(brain.getRID())
        self.assertEqual(indexes.get("latitude"), "")
        self.assertEqual(indexes.get("longitude"), "")

        api.content.delete(obj=section)
        brain = api.content.find(UID=uuid)[0]
        indexes = catalog.getIndexDataForRID(brain.getRID())
        self.assertEqual(indexes.get("latitude"), 4.5)
        self.assertEqual(indexes.get("longitude"), 45)

        api.content.delete(obj=section2)
        brain = api.content.find(UID=uuid)[0]
        indexes = catalog.getIndexDataForRID(brain.getRID())
        self.assertEqual(indexes.get("latitude"), "")
        self.assertEqual(indexes.get("longitude"), "")

    def test_section_error(self):
        page = api.content.create(
            container=self.portal,
            type="imio.smartweb.Page",
            title="Page",
        )
        api.content.transition(page, "publish")
        api.content.create(
            container=page,
            type="imio.smartweb.SectionLinks",
            title="Section links",
        )
        view = getMultiAdapter((page, self.request), name="full_view")
        self.assertNotIn("Error in section :", view())
        with patch(
            "imio.smartweb.core.contents.sections.links.view.LinksView.items",
            side_effect=Exception,
        ):
            self.assertIn('Error in section : "Section links"', view())
            logout()
            self.assertNotIn("Error in section :", view())



# -*- coding: utf-8 -*-

from collective.instancebehavior.interfaces import IInstanceBehaviorAssignableContent
from imio.smartweb.core.contents import IPages
from imio.smartweb.core.contents import Pages
from zope.interface import implementer


class IPage(IPages):
    """Marker interface and Dexterity Python Schema for Page"""


@implementer(IPage, IInstanceBehaviorAssignableContent)
class Page(Pages):
    """Page class"""

    category_name = "page_category"



# -*- coding: utf-8 -*-

from collective.taxonomy.interfaces import ITaxonomy
from imio.smartweb.core.browser.instancebehaviors.form import InstanceBehaviors
from imio.smartweb.core.testing import IMIO_SMARTWEB_CORE_FUNCTIONAL_TESTING
from imio.smartweb.core.testing import ImioSmartwebTestCase
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import TEST_USER_PASSWORD
from plone.app.z3cform.interfaces import IPloneFormLayer
from plone.testing.zope import Browser
from zope.component import getMultiAdapter
from zope.component import queryUtility
from zope.interface import alsoProvides
from zope.i18n.interfaces import ITranslationDomain
from zope.schema.interfaces import IVocabularyFactory

import transaction


class TestInstanceBehaviors(ImioSmartwebTestCase):
    layer = IMIO_SMARTWEB_CORE_FUNCTIONAL_TESTING

    def setUp(self):
        self.authorized_instance_behaviors = [
            "imio.smartweb.Page",
            "imio.smartweb.Procedure",
        ]
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.folder = api.content.create(
            container=self.portal,
            type="imio.smartweb.Folder",
            id="folder",
        )
        self.page = api.content.create(
            container=self.folder,
            type="imio.smartweb.Page",
            id="page",
        )
        taxonomy = queryUtility(ITaxonomy, name="collective.taxonomy.test")
        self.assertIsNotNone(taxonomy)

    def delete_taxonomy(self, name):
        sm = self.portal.getSiteManager()
        utility = sm.queryUtility(ITaxonomy, name=name)
        utility.unregisterBehavior()
        sm.unregisterUtility(utility, ITaxonomy, name=name)
        sm.unregisterUtility(utility, IVocabularyFactory, name=name)
        sm.unregisterUtility(utility, ITranslationDomain, name=name)

    def test_action_available(self):
        for t in self.authorized_instance_behaviors:
            obj = api.content.create(
                container=self.folder,
                type=t,
                title="My {}".format(t),
            )
            is_instancebehaviors_assignable_content = getMultiAdapter(
                (obj, self.request), name="is_instancebehaviors_assignable_content"
            )
            self.assertTrue(is_instancebehaviors_assignable_content())

        is_instancebehaviors_assignable_content = getMultiAdapter(
            (self.folder, self.request), name="is_instancebehaviors_assignable_content"
        )
        self.assertFalse(is_instancebehaviors_assignable_content())

        is_instancebehaviors_assignable_content = getMultiAdapter(
            (self.page, self.request), name="is_instancebehaviors_assignable_content"
        )
        self.delete_taxonomy("collective.taxonomy.test")
        self.assertFalse(is_instancebehaviors_assignable_content())

    def test_form(self):
        transaction.commit()
        browser = Browser(self.layer["app"])
        browser.addHeader(
            "Authorization",
            "Basic %s:%s"
            % (
                TEST_USER_NAME,
                TEST_USER_PASSWORD,
            ),
        )
        browser.open("{}/edit".format(self.page.absolute_url()))
        content = browser.contents
        self.assertNotIn("form-widgets-test-taxonomy_test", content)
        request = self.request
        request.form = {
            "form.widgets.instance_behaviors": "collective.taxonomy.generated.test",
            "form.buttons.ok": "Ok",
        }
        alsoProvides(request, IPloneFormLayer)
        form = InstanceBehaviors(self.page, request)
        form.update()
        transaction.commit()
        browser.open("{}/edit".format(self.page.absolute_url()))
        content = browser.contents
        self.assertIn("form-widgets-test-taxonomy_test", content)

    def test_vocabulary(self):
        vocabulary_name = "imio.smartweb.vocabulary.AvailableInstanceBehaviors"
        self.assertVocabularyLen(vocabulary_name, 1)
        self.delete_taxonomy("collective.taxonomy.test")
        self.assertVocabularyLen(vocabulary_name, 0)



# -*- coding: utf-8 -*-

from Acquisition import aq_parent
from imio.smartweb.core.contents import IFolder
from imio.smartweb.core.utils import get_default_content_id
from imio.smartweb.locales import SmartwebMessageFactory as _
from plone import api
from plone.app.layout.navigation.interfaces import INavigationRoot
from Products.CMFPlone.interfaces.constrains import DISABLED
from Products.CMFPlone.interfaces.constrains import ISelectableConstrainTypes
from Products.Five.browser import BrowserView


class HeroBannerSettings(BrowserView):
    """HeroBanner settings"""

    def add_herobanner(self):
        if not self.available():
            api.portal.show_message(
                _("Hero banner cannot be added here"), self.request, type="error"
            )
            self.request.response.redirect(self.context.absolute_url())
            return ""
        obj = self.context
        if not INavigationRoot.providedBy(obj):
            obj = aq_parent(obj)
        if IFolder.providedBy(obj):
            container = ISelectableConstrainTypes(obj)
            constrain_types_mode = container.getConstrainTypesMode()
            container.setConstrainTypesMode(DISABLED)
        pt = api.portal.get_tool("portal_types")
        portal_type = obj.portal_type
        allowed_content_types = pt.getTypeInfo(portal_type).allowed_content_types
        allowed_content_types = list(allowed_content_types)
        allowed_content_types.append("imio.smartweb.HeroBanner")
        pt.getTypeInfo(portal_type).allowed_content_types = tuple(allowed_content_types)
        herobanner = api.content.create(
            title=_("Hero banner"),
            id="herobanner",
            container=obj,
            type="imio.smartweb.HeroBanner",
        )
        herobanner.exclude_from_parent_listing = True
        herobanner.reindexObject(idxs=["exclude_from_parent_listing"])
        allowed_content_types.remove("imio.smartweb.HeroBanner")
        pt.getTypeInfo(portal_type).allowed_content_types = tuple(allowed_content_types)
        if IFolder.providedBy(obj):
            container.setConstrainTypesMode(constrain_types_mode)
        api.portal.show_message(
            _("Hero banner has been successfully added"), self.request
        )
        self.request.response.redirect(herobanner.absolute_url())

    def available(self):
        obj = self.context
        if not INavigationRoot.providedBy(obj):
            parent = aq_parent(obj)
            if not INavigationRoot.providedBy(parent):
                return False

            default_page_id = get_default_content_id(parent)
            if default_page_id == obj.id:
                obj = parent
            else:
                return False

        herobanners = obj.listFolderContents(
            contentFilter={"portal_type": "imio.smartweb.HeroBanner"}
        )
        return len(herobanners) == 0
